/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/* $Id: audio.c,v 1.6 2003/04/12 20:44:30 dolphy Exp $ */

#include "getcontrol.h"

image_node * audio_menu_images_list = NULL;
image_node * streams_menu_images_list = NULL;

/* load_audio_menu
	Function which loads the audio menu's pictures in
	an image list memory structure */
	
gboolean
load_audio_menu (void) {
	gint i, image_coord = 0, ang_step;
	gint nb_activities;
	GError * error = NULL;
	image_node * new_image;
	
	gchar * activities_names[] = {
		gettext_noop ("from_stream"),
		gettext_noop ("from_cd"),
		gettext_noop ("from_hd"),
		gettext_noop ("from_nfs"),
		gettext_noop ("back")
	};
	
	remove_all_timers ();
	
	if (audio_menu_images_list) { /* Freeing already loaded pics */
		free_images_list (audio_menu_images_list);
		audio_menu_images_list = NULL;
	}
	
	/* How many activities do we have */
	
	nb_activities = sizeof (activities_names) / sizeof (activities_names[0]);
	
	ang_step = 360 / nb_activities;
	
	for (i = 0; i < nb_activities; i++) {
			
		new_image = (image_node *) malloc (sizeof(image_node));
			
		new_image->image_path = g_strdup_printf (
									"%s"G_DIR_SEPARATOR_S"%s.png",
									current_theme,
									activities_names[i]);
				
		new_image->image = gdk_pixbuf_new_from_file (new_image->image_path, &error);
			
		if (!new_image->image) {
			g_message ("failed to load image %s", new_image->image_path);
			return FALSE;
		}
		
		if (!i) {
			image_coord = 0; /* First picture at 0 degree */
			new_image->selected = TRUE; /* Is image selected */
		}
		else {
			image_coord = image_coord + ang_step; /* Changing image_coord */
			new_image->selected = FALSE; /* Is image selected */
		}
		
		new_image->image_label = g_strdup(gettext(activities_names[i]));
		
		new_image->xpos = new_image->x_offset = 0;
		
		new_image->ypos = new_image->y_offset = 0;
		
		new_image->width = gdk_pixbuf_get_width (new_image->image);
		
		new_image->height = gdk_pixbuf_get_height (new_image->image);
		
		new_image->alpha_level = 255; /* Alpha level to maximum */
		
		new_image->zoom_ratio = new_image->init_zoom_ratio = 1; /* Zoom ratio full size */
		
		new_image->visible = TRUE; /* Visible by default */
		
		new_image->angle = image_coord + ANG_OFFSET; /* Initial menu position */
				
		if (new_image->angle > 360)
			new_image->angle = new_image->angle - 360; /* Staying between 0-360 */
		
		new_image->data = NULL;
		
		switch (i) {
				case 0 :
					new_image->image_link = (GtkFunction) activate_audio_from_stream;
					if ((media_play) &&
						(media_play->audio_play_source == FROM_STREAM) &&
						(	gst_play_get_state(media_play->audio_play) !=
							GST_STATE_READY) )
						new_image->busy = TRUE;
					else
						new_image->busy = FALSE;
					break;;
				case 1 :
					new_image->image_link = (GtkFunction) activate_audio_from_cd;
					if ((media_play) &&
						(media_play->audio_play_source == FROM_CD) &&
						(	gst_play_get_state(media_play->audio_play) !=
							GST_STATE_READY) )
						new_image->busy = TRUE;
					else
						new_image->busy = FALSE;
					break;;
				case 2 :
					new_image->image_link = (GtkFunction) activate_audio_from_hd;
					if ((media_play) &&
						(media_play->audio_play_source == FROM_HD) &&
						(	gst_play_get_state(media_play->audio_play) !=
							GST_STATE_READY) )
						new_image->busy = TRUE;
					else
						new_image->busy = FALSE;
					break;;
				case 3 :
					new_image->image_link = (GtkFunction) activate_audio_from_nfs;
					if ((media_play) &&
						(media_play->audio_play_source == FROM_NFS) &&
						(	gst_play_get_state(media_play->audio_play) !=
							GST_STATE_READY) )
						new_image->busy = TRUE;
					else
						new_image->busy = FALSE;
					break;;
				case 4 :
					new_image->image_link = (GtkFunction) activate_main_menu;
					new_image->busy = FALSE;
					break;;
		}
		
		new_image->next_image = NULL;
		
		add_image_at_the_end (new_image, &audio_menu_images_list);
	
	}
	
	return TRUE;
}
gboolean
activate_audio_menu (gpointer data)
{
	if (load_audio_menu ()) {
		current_menu_images_list = audio_menu_images_list;
		current_menu = AUDIO_MENU;
	}
	
	draw_menu (current_menu_images_list);
	
	return TRUE;
}


gboolean
play_audio_from_stream (gpointer data)
{
	
	play_audio ((gchar *)data, FROM_STREAM);
	
	return TRUE;
	
}

/* load_streams_menu
	Function taking care of loading the streams choice menu
	according to XML streams list
*/

gboolean
load_streams_menu (void)
{
	gint i, image_coord = 0, ang_step;
	gint nb_activities;
	GError * error = NULL;
	image_node * new_image;
	gchar * path_to_streams_file = NULL, * temp_string;
	xmlconfig * streams_config = NULL;
	
	remove_all_timers ();
	
	if (streams_menu_images_list) { /* Freeing already loaded pics */
		free_images_list (streams_menu_images_list);
		streams_menu_images_list = NULL;
	}
	
	path_to_streams_file = g_build_path ( G_DIR_SEPARATOR_S,
								g_get_home_dir (),
								".getcontrol",
								"streams.xml",NULL);
	
	/* If config file found in user home dir we ignore system one */
	
	if (!g_file_test (path_to_streams_file, G_FILE_TEST_EXISTS)) {
		g_free(path_to_streams_file);
		path_to_streams_file = g_build_path (G_DIR_SEPARATOR_S,
									DATADIR,
									"getcontrol",
									"streams.xml", NULL);
		if (!g_file_test (path_to_streams_file, G_FILE_TEST_EXISTS)) {
			g_warning ("unable to find a streams config file");
			return FALSE;
		}
	}
	
	streams_config = xmlconfig_open(path_to_streams_file);
	
	if (path_to_streams_file) g_free(path_to_streams_file);

	if (!streams_config) {
		g_warning ("failed to open xml config file for streams");
		return FALSE;
	}
	
	/* How many activities do we have */
	
	temp_string = NULL;
	
	temp_string = xmlconfig_get_attribute (streams_config, "/streams", "number");
	
	if (!temp_string) {
		g_warning("failed to get the number of streams");
		if (streams_config)
			xmlconfig_close (streams_config);
		return FALSE;
	}
	
	nb_activities = atoi (temp_string);
	
	if (temp_string) g_free (temp_string);
	
	ang_step = 360 / nb_activities;
	
	for (i = 0; i < nb_activities; i++) {
		
		gchar * xpath = NULL;
		
		new_image = (image_node *) malloc (sizeof(image_node));
		
		xpath = g_build_path ("/", "streams", "stream", NULL); // FIXME
		
		new_image->image_path = xmlconfig_get_attribute_pos (
									streams_config, xpath, "image", i);

		if (!new_image->image_path) {
			g_warning ("failed to get image path from streams.xml");
			if (streams_config) xmlconfig_close (streams_config);
			return FALSE;
		}
		
		new_image->image = gdk_pixbuf_new_from_file (new_image->image_path, &error);
			
		if (!new_image->image) {
			g_message ("failed to load image %s", new_image->image_path);
			if (streams_config) xmlconfig_close (streams_config);
			return FALSE;
		}
		
		if (!i) {
			image_coord = 0; /* First picture at 0 degree */
			new_image->selected = TRUE; /* Is image selected */
		}
		else {
			image_coord = image_coord + ang_step; /* Changing image_coord */
			new_image->selected = FALSE; /* Is image selected */
		}
		
		new_image->image_label = xmlconfig_get_attribute_pos (
									streams_config, xpath, "label", i);
		
		if (!new_image->image_label) {
			g_warning ("failed to get image label from streams.xml");
			if (streams_config) xmlconfig_close (streams_config);
			return FALSE;
		}
		
		new_image->xpos = 0;
		
		new_image->ypos = 0;
		
		new_image->width = gdk_pixbuf_get_width (new_image->image);
		
		new_image->height = gdk_pixbuf_get_height (new_image->image);
		
		new_image->alpha_level = 255; /* Alpha level to maximum */
		
		new_image->zoom_ratio = new_image->init_zoom_ratio = 1; /* Zoom ratio full size */
		
		new_image->visible = TRUE; /* Visible by default */
		
		new_image->angle = image_coord + ANG_OFFSET; /* Initial menu position */
				
		if (new_image->angle > 360)
			new_image->angle = new_image->angle - 360; /* Staying between 0-360 */
		
		new_image->image_link = (GtkFunction) play_audio_from_stream;
		
		new_image->data = (gchar *) xmlconfig_get_attribute_pos (
										streams_config, xpath, "url", i);
		
		if ((media_play) &&
			(media_play->audio_location) &&
			(g_strcasecmp (new_image->data, media_play->audio_location) == 0) )
				new_image->busy = TRUE;
		else
			new_image->busy = FALSE;
		
		if (xpath) g_free (xpath);
		
		xpath = NULL;
		
		if (!new_image->data) {
			g_warning ("failed to get image label from streams.xml");
			if (streams_config) xmlconfig_close (streams_config);
			return FALSE;
		}
			
		new_image->next_image = NULL;
		
		add_image_at_the_end (new_image, &streams_menu_images_list);
	
	}
	
	if (streams_config)
		xmlconfig_close (streams_config);
	
	return TRUE;	
}

gboolean
activate_audio_from_stream (gpointer data)
{
	
	if (load_streams_menu ()) {
		current_menu_images_list = streams_menu_images_list;
		current_menu = STREAMS_MENU;
	}
	
	draw_menu (current_menu_images_list);
	
	return TRUE;
}


gboolean
activate_audio_from_cd (gpointer data)
{
	
	play_audio("/tmp/south.mp3", FROM_CD);
	
	return TRUE;
}

gboolean
activate_audio_from_hd (gpointer data)
{
	
	play_audio("/tmp/south.mp3", FROM_HD);
	
	return TRUE;
}

gboolean
activate_audio_from_nfs (gpointer data)
{
	
	play_audio("/tmp/south.mp3", FROM_NFS);
	
	return TRUE;
}

void
toggle_visualisation ()
{
	if (media_play == NULL)
		return;
	
	g_return_if_fail (media_play->audio_play != NULL);
	g_return_if_fail (GST_IS_PLAY(media_play->audio_play));
	
	if ( gst_play_get_state (media_play->audio_play) != GST_STATE_READY ) {
		if (visualisation) {
			remove_all_timers();
			gtk_widget_hide(da);
			gtk_widget_show(media_play->vw);
			current_status = A_PLAYING;
		}
		else {
			gtk_widget_hide(media_play->vw);
			gtk_widget_show(da);
			activate_main_menu (main_menu_images_list);
		}
		gst_play_connect_visualisation (media_play->audio_play, visualisation);
	}
	
}
