/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* $Id: callbacks.c,v 1.31 2003/04/12 20:44:30 dolphy Exp $ */
 
#include "getcontrol.h"

/* Callbacks */

gboolean
getcontrol_quit_callback (GtkWidget * widget, gpointer data)
{	
	remove_all_timers ();
	
	if (frame) /* Freeing all our pixbufs */
		gdk_pixbuf_unref (frame);
	if (frame_backup)
		gdk_pixbuf_unref (frame_backup);
	if (background)
		gdk_pixbuf_unref (background);
	if (background_orig)
		gdk_pixbuf_unref (background_orig);
	
	if (main_menu_images_list)
		free_images_list (main_menu_images_list);
	
	if (prefs_menu_images_list)
		free_images_list (prefs_menu_images_list);
		
	if (themes_menu_images_list)
		free_images_list (themes_menu_images_list);
	
	if (screensaver_menu_images_list)
		free_images_list (screensaver_menu_images_list);
		
	if (anim_menu_images_list)
		free_images_list (anim_menu_images_list);
	
	if (current_theme) g_free(current_theme);
	
	if (selected_label) g_free(selected_label);
	
	if (font_markup) g_free (font_markup);
	
	gtk_main_quit ();
	
}


gboolean
getcontrol_about_callback (GtkWidget * widget, gpointer data)
{
	GdkPixbuf *pixbuf = NULL;
	static GtkWidget *about = NULL;
	const gchar *authors[] = {
		"Julien MOUTTE. <julien@moutte.net>",
		"Matthias SAOU. <matthias.saou@est.une.marmotte.net>",
		NULL
	};

	/* Documenters credits */

	gchar *documenters[] = {
		NULL
	};

	/* Translator credits */

	gchar *translator_credits = _("translator_credits");
	
	if (about) {
		gtk_window_present (GTK_WINDOW (about));
		return;
	}

	{
		char *filename = NULL;
		filename = gnome_program_locate_file (
				NULL,
				GNOME_FILE_DOMAIN_APP_PIXMAP,
				("getcontrol.png"),
				TRUE, NULL);

		if (filename != NULL)
		{
			pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
			g_free (filename);
		}
	}

	about = gnome_about_new (
			_("Getcontrol"), VERSION,
			"(C) 2002 the Free Software Foundation",
			_("The front end you need for multimedia boxes."),
			(const char **)authors,
			(const char **)documenters,
			strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
			pixbuf
				);

	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (app));
	gtk_widget_show (about);
	g_signal_connect (
			G_OBJECT (about), "destroy",
			G_CALLBACK (gtk_widget_destroyed), &about);

	return TRUE;
}

/* Expose callback for the drawing area */
gboolean
expose_callback (GtkWidget * widget, GdkEventExpose * event, gpointer data)
{
	guchar *pixels;
	int rowstride;
	guint text_xpos = 0, text_ypos = 0;
	PangoLayout * getcontrol_plyt = NULL;
	PangoAttrList * getcontrol_attrlist = NULL;
	gchar * parsed_label = NULL, * marked_up_label = NULL;
		
	rowstride = gdk_pixbuf_get_rowstride (frame);

	pixels = gdk_pixbuf_get_pixels (frame) + rowstride * event->area.y + event->area.x * 3;

	gdk_draw_rgb_image_dithalign (
			widget->window,
			widget->style->black_gc,
			event->area.x, event->area.y,
			event->area.width, event->area.height,
			GDK_RGB_DITHER_NORMAL,
			pixels, rowstride,
			event->area.x, event->area.y);
	
	if (current_status == SELECTED) {
		
		marked_up_label = g_strdup_printf ("%s%s</span>",
							font_markup, selected_label);
		
		pango_parse_markup (
			marked_up_label, -1, 0,
			&getcontrol_attrlist,
			&parsed_label,
			NULL, NULL);
		
		if (marked_up_label) g_free (marked_up_label);
	
		getcontrol_plyt = gtk_widget_create_pango_layout (da, parsed_label);
		
		pango_layout_set_attributes (getcontrol_plyt, getcontrol_attrlist);
		
		pango_layout_get_pixel_size (getcontrol_plyt, &text_xpos, &text_ypos);
	
		text_xpos = xmid - (text_xpos / 2);

		/* Trying to put text in the middle of available space */
		
		text_ypos = (((ymid * 2) - (ymid+radius)) / 2) + (ymid+radius) - (text_ypos / 2);
	
		gdk_draw_layout (
			widget->window,
			widget->style->black_gc,
			text_xpos, text_ypos,
			getcontrol_plyt);
			
		if (parsed_label)
			g_free (parsed_label);
	}
		
	return TRUE;
}

/* Callback for key_press_event management */
gboolean
key_pressed_callback (GtkWidget * widget, GdkEventKey * event, image_node * images_list)
{
	image_node * l_images_list = NULL;
	image_node * new_image = NULL;
	
	switch (event->keyval) {
		case 65361 : /* LEFT (ROTATE LEFT) */
			switch (current_status) {
				case IDLE :
				case SELECTED :
					remove_all_timers ();
					direction = LEFT;
					menu_rotation_manager (current_menu_images_list);
					return TRUE;
				case SCREEN_SAVING :
					remove_all_timers ();
					draw_menu (current_menu_images_list);
					return TRUE;
				case ROTATING :
					stack_order (&order_queue, LEFT, ROTATE_ORDER);
					return TRUE;
				default :
					return TRUE;
			}
			break;;
		case 65363 : /* RIGHT (ROTATE RIGHT) */
			switch (current_status) {
				case IDLE :
				case SELECTED :
					remove_all_timers ();
					direction = RIGHT;
					menu_rotation_manager (current_menu_images_list);
					return TRUE;
				case SCREEN_SAVING :
					remove_all_timers ();
					draw_menu (current_menu_images_list);
					return TRUE;
				case ROTATING :
					stack_order (&order_queue, RIGHT, ROTATE_ORDER);
					return TRUE;
				default :
					return TRUE;
			}
			break;;
		case GDK_space : /* SPACE (FULLSCREEN) */
			if (fullscreen)
				fullscreen = FALSE;
			else
				fullscreen = TRUE;
			
			/* Storing the change in GConf */
			gconf_client_set_bool (conf_client,
						FULLSCREEN_PATH, fullscreen, NULL);
			return TRUE;
			break;;
		case GDK_Return : /* ENTER (VALIDATE) */
			switch (current_status) {
				case IDLE :
				case SELECTED :
					remove_all_timers ();
					l_images_list = current_menu_images_list;
					while ( !(l_images_list->selected) && (l_images_list) )
						l_images_list = l_images_list->next_image;
					l_images_list->image_link (l_images_list->data);
					return TRUE;
				case SCREEN_SAVING :
					remove_all_timers ();
					draw_menu (current_menu_images_list);
					return TRUE;
				case ROTATING :
					stack_order (&order_queue, LEFT, ACTIVATE_ORDER);
					return TRUE;
				default :
					return TRUE;
			}
			break;;
		case 65288 : /* BACKSPACE (MAIN MENU) */
			switch (current_status) {
				case V_PLAYING :
					gst_play_set_state (	media_play->video_play,
											GST_STATE_PAUSED);
					if (GTK_WIDGET_VISIBLE(media_play->vw))
						gtk_widget_hide (GTK_WIDGET (media_play->vw));
					
					gtk_widget_show (da);
					
					gtk_widget_grab_focus ( GTK_WIDGET( da) );
				
					/* Restoring Audio playback if paused */
					
					if ((GST_IS_PLAY(media_play->audio_play)) &&
						(	gst_play_get_state (media_play->audio_play) ==
							GST_STATE_PAUSED ))
					{
						gst_play_set_state (	media_play->audio_play,
												GST_STATE_PLAYING);
						media_play->current_play = media_play->audio_play;
					}
					
					activate_main_menu (main_menu_images_list);
					break;;
				case A_PLAYING :
					if (GTK_WIDGET_VISIBLE(media_play->vw))
						gtk_widget_hide (GTK_WIDGET (media_play->vw));
					
					gtk_widget_show (da);
					
					gtk_widget_grab_focus ( GTK_WIDGET( da) );
				default :
					g_message ("main menu");
					activate_main_menu (main_menu_images_list);
			}
			return TRUE;
			break;;
		case GDK_l : /* L (PLAY) */
			start_playback();
			return TRUE;
			break;;
		case GDK_p : /* P (PAUSE) */
			toggle_pause_playback ();
			return TRUE;
			break;;
		case GDK_s : /* S (STOP) */
			switch (current_status) {
				case V_PLAYING :
					if ((	gst_play_get_state (media_play->video_play) ==
							GST_STATE_PLAYING) ||
						(	gst_play_get_state (media_play->video_play) ==
							GST_STATE_PAUSED))
						gst_play_eos (media_play->video_play);
				break;;
				default :
					if ((	gst_play_get_state (media_play->audio_play) ==
							GST_STATE_PLAYING) ||
						(	gst_play_get_state (media_play->audio_play) ==
							GST_STATE_PAUSED))
						gst_play_eos (media_play->audio_play);
				break;;
			}
			return TRUE;
			break;;
		case GDK_m : /* M (MUTE) */
			toggle_mute_playback ();
			return TRUE;
			break;;
		case GDK_e : /* E (EJECT) */
			
			remove_all_timers();
		
			new_image = (image_node *) malloc (sizeof (image_node));
		
			new_image->image_path = g_strdup_printf (
										"%s"G_DIR_SEPARATOR_S"eject.png",
										current_theme);
		
			new_image->image = gdk_pixbuf_new_from_file (new_image->image_path, NULL);
		
			new_image->selected = FALSE; /* Is image selected */
		
			new_image->image_label = g_strdup("Eject");
		
			new_image->width = gdk_pixbuf_get_width (new_image->image);
		
			new_image->height = gdk_pixbuf_get_height (new_image->image);
		
			new_image->xpos = xmid - (new_image->width / 2);
		
			new_image->x_offset = 0;
		
			new_image->ypos = ymid - (new_image->height / 2);
			
			new_image->y_offset = 0;
		
			new_image->alpha_level = 255; /* Alpha level to maximum */
		
			new_image->zoom_ratio = new_image->init_zoom_ratio = 1; /* Zoom ratio full size */
		
			new_image->visible = TRUE; /* Visible by default */
			
			new_image->busy = FALSE; /* Not busy by default */
			
			new_image->data = NULL;
			
			new_image->next_image = NULL;
		
			add_image_at_the_end (new_image, &current_menu_images_list);
			
			draw_images (current_menu_images_list);
			
			sleep (1);
			
			if (	(media_play) &&
					(media_play->video_play_source == FROM_CD)) {
				gst_play_eos (media_play->video_play);
			}
			else if (	(media_play) &&
					(media_play->audio_play_source == FROM_CD)) {
				gst_play_eos (media_play->audio_play);
			}
			else {
				remove_last_image (&current_menu_images_list);
				draw_images (current_menu_images_list);
			}
			eject_cdrom ();
			
			return TRUE;
			break;;
		case GDK_KP_0 : /* 0 ZERO */
			if (	(current_status == V_PLAYING) &&
					(GST_IS_PLAY(media_play->video_play)) )
				gst_play_seek_to_time (media_play->video_play, 0);
			else if (GST_IS_PLAY (media_play->audio_play))
				gst_play_seek_to_time (media_play->audio_play, 0);
			return TRUE;
			break;;
		case GDK_KP_1 : /* 1 ONE */
			if (	(current_status == V_PLAYING) &&
					(GST_IS_PLAY(media_play->video_play)) )
				gst_play_seek_to_time (media_play->video_play,
					(10LL * media_play->video_length_nanos) / 100LL);
			else if (GST_IS_PLAY (media_play->audio_play))
				gst_play_seek_to_time (media_play->audio_play,
					(10LL * media_play->audio_length_nanos) / 100LL);
			return TRUE;
			break;;
		case GDK_KP_2 : /* 2 TWO */
			if (	(current_status == V_PLAYING) &&
					(GST_IS_PLAY(media_play->video_play)) )
				gst_play_seek_to_time (media_play->video_play,
					(20LL * media_play->video_length_nanos) / 100LL);
			else if (GST_IS_PLAY (media_play->audio_play))
				gst_play_seek_to_time (media_play->audio_play,
					(20LL * media_play->audio_length_nanos) / 100LL);
			return TRUE;
			break;;
		case GDK_KP_3 : /* 3 THREE */
			if (	(current_status == V_PLAYING) &&
					(GST_IS_PLAY(media_play->video_play)) )
				gst_play_seek_to_time (media_play->video_play,
					(30LL * media_play->video_length_nanos) / 100LL);
			else if (GST_IS_PLAY (media_play->audio_play))
				gst_play_seek_to_time (media_play->audio_play,
					(30LL * media_play->audio_length_nanos) / 100LL);
			return TRUE;
			break;;
		case GDK_KP_4 : /* 4 FOUR */
			if (	(current_status == V_PLAYING) &&
					(GST_IS_PLAY(media_play->video_play)) )
				gst_play_seek_to_time (media_play->video_play,
					(40LL * media_play->video_length_nanos) / 100LL);
			else if (GST_IS_PLAY (media_play->audio_play))
				gst_play_seek_to_time (media_play->audio_play,
					(40LL * media_play->audio_length_nanos) / 100LL);
			return TRUE;
			break;;
		case GDK_KP_5 : /* 5 FIVE */
			if (	(current_status == V_PLAYING) &&
					(GST_IS_PLAY(media_play->video_play)) )
				gst_play_seek_to_time (media_play->video_play,
					(50LL * media_play->video_length_nanos) / 100LL);
			else if (GST_IS_PLAY (media_play->audio_play))
				gst_play_seek_to_time (media_play->audio_play,
					(50LL * media_play->audio_length_nanos) / 100LL);
			return TRUE;
			break;;
		case GDK_KP_6 : /* 6 SIX */
			if (	(current_status == V_PLAYING) &&
					(GST_IS_PLAY(media_play->video_play)) )
				gst_play_seek_to_time (media_play->video_play,
					(60LL * media_play->video_length_nanos) / 100LL);
			else if (GST_IS_PLAY (media_play->audio_play))
				gst_play_seek_to_time (media_play->audio_play,
					(60LL * media_play->audio_length_nanos) / 100LL);
			return TRUE;
			break;;
		case GDK_KP_7 : /* 7 SEVEN */
			if (	(current_status == V_PLAYING) &&
					(GST_IS_PLAY(media_play->video_play)) )
				gst_play_seek_to_time (media_play->video_play,
					(70LL * media_play->video_length_nanos) / 100LL);
			else if (GST_IS_PLAY (media_play->audio_play))
				gst_play_seek_to_time (media_play->audio_play,
					(70LL * media_play->audio_length_nanos) / 100LL);
			return TRUE;
			break;;
		case GDK_KP_8 : /* 8 EIGHT */
			if (	(current_status == V_PLAYING) &&
					(GST_IS_PLAY(media_play->video_play)) )
				gst_play_seek_to_time (media_play->video_play,
					(80LL * media_play->video_length_nanos) / 100LL);
			else if (GST_IS_PLAY (media_play->audio_play))
				gst_play_seek_to_time (media_play->audio_play,
					(80LL * media_play->audio_length_nanos) / 100LL);
			return TRUE;
			break;;
		case GDK_KP_9 : /* 9 NINE */
			if (	(current_status == V_PLAYING) &&
					(GST_IS_PLAY(media_play->video_play)) )
				gst_play_seek_to_time (media_play->video_play,
					(90LL * media_play->video_length_nanos) / 100LL);
			else if (GST_IS_PLAY (media_play->audio_play))
				gst_play_seek_to_time (media_play->audio_play,
					(90LL * media_play->audio_length_nanos) / 100LL);
			return TRUE;
			break;;
		case GDK_KP_Add : /* + VOL+ */
			if (	(current_status == V_PLAYING) &&
					(GST_IS_PLAY(media_play->video_play)) )
				set_volume (media_play->video_play, 
					gst_play_get_volume (media_play->video_play) + 0.1);
			if ( GST_IS_PLAY(media_play->audio_play) )
				set_volume (media_play->audio_play, 
					gst_play_get_volume (media_play->audio_play) + 0.1);
			return TRUE;
			break;;
		case GDK_KP_Subtract : /* - VOL- */
			if (	(current_status == V_PLAYING) &&
					(GST_IS_PLAY(media_play->video_play)) )
				set_volume (media_play->video_play, 
					gst_play_get_volume (media_play->video_play) - 0.1);
			if ( GST_IS_PLAY(media_play->audio_play) )
				set_volume (media_play->audio_play, 
					gst_play_get_volume (media_play->audio_play) - 0.1);
			return TRUE;
			break;;
		default : /* UNRECOGNIZED */
			g_message ("unrecognized key pressed : %d\n", event->keyval);
	}
	
	return TRUE;

}

/* Callback for resizing and first drawing */
gboolean
configure_callback (GtkWidget * widget, GdkEventConfigure *event)
{

	gdouble width_ratio_scale, height_ratio_scale;
	
	remove_all_timers ();
	
	if (frame) /* Freeing all our pixbufs */
		gdk_pixbuf_unref (frame);
	if (frame_backup)
		gdk_pixbuf_unref (frame_backup);
	if (background)
		gdk_pixbuf_unref (background);
	
	draw_width = widget->allocation.width;
	draw_height = widget->allocation.height;

	/* Creating our pixbufs */
	
	frame = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
							FALSE, 8, draw_width,
							draw_height);
		
	frame_backup = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
							FALSE, 8, draw_width,
							draw_height);

	background = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
							FALSE, 8, draw_width,
							draw_height);
	
	/* Middle of the picture */
	
	xmid = draw_width / 2.0;
	ymid = draw_height / 2.0;

	/* Calculating radius */

	radius = MIN (xmid, ymid) / 2.0;
	
	width_ratio_scale = (gdouble) draw_width / (gdouble) back_width;
	height_ratio_scale = (gdouble) draw_height / (gdouble) back_height;
	
	/* Drawing & scaling our background */
	
	gdk_pixbuf_composite (
		background_orig,
		background,
		0, 0,
		draw_width, draw_height,
		0, 0,
		width_ratio_scale, height_ratio_scale,
		GDK_INTERP_BILINEAR,
		255);

	/* Drawing menu */
	
	draw_menu (current_menu_images_list);
	
	return TRUE;
}

gboolean
remove_all_timers ()
{
	if (select_timeout_id) /* Removing selection timer */
		gtk_timeout_remove (select_timeout_id);
	
	select_timeout_id = 0;
	
	if (screen_saver_timeout_id) /* Removing screen saver timer */
		gtk_timeout_remove (screen_saver_timeout_id);
	
	screen_saver_timeout_id = 0;
		
	if (draw_timeout_id) /* Removing animation timer */
		gtk_timeout_remove (draw_timeout_id);
	
	draw_timeout_id = 0;
	
	if (ss_draw_timeout_id)
		gtk_timeout_remove (ss_draw_timeout_id);
	
	ss_draw_timeout_id = 0;
	
	if (busy_draw_timeout_id)
		gtk_timeout_remove (busy_draw_timeout_id);
	
	busy_draw_timeout_id = 0;
	
	if (ss_bg_switch_timeout_id)
		gtk_timeout_remove (ss_bg_switch_timeout_id);
	
	ss_bg_switch_timeout_id = 0;
	
	return TRUE;
}


/* This is called when our timer keys are changed. */
void
gconf_notify_global (GConfClient *client,
		   guint        cnxn_id,
		   GConfEntry  *entry,
		   gpointer     user_data)
{
		load_config ();
}

