/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/* $Id: divx.c,v 1.7 2003/02/23 17:54:38 dolphy Exp $ */

#include "getcontrol.h"

image_node * divx_menu_images_list = NULL;

/* load_divx_menu
	Function which loads the divx menu's pictures in
	an image list memory structure */
	
gboolean
load_divx_menu (void) {
	gint i, image_coord = 0, ang_step;
	gint nb_activities;
	GError * error = NULL;
	image_node * new_image;
	
	gchar * activities_names[] = {
		gettext_noop ("from_cd"),
		gettext_noop ("from_hd"),
		gettext_noop ("from_nfs"),
		gettext_noop ("back")
	};
	
	remove_all_timers ();
	
	if (divx_menu_images_list) { /* Freeing already loaded pics */
		free_images_list (divx_menu_images_list);
		divx_menu_images_list = NULL;
	}
	
	/* How many activities do we have */
	
	nb_activities = sizeof (activities_names) / sizeof (activities_names[0]);
	
	ang_step = 360 / nb_activities;
	
	for (i = 0; i < nb_activities; i++) {
			
		new_image = (image_node *) malloc (sizeof(image_node));
			
		new_image->image_path = g_strdup_printf (
									"%s/%s.png",
									current_theme,
									activities_names[i]);
				
		new_image->image = gdk_pixbuf_new_from_file (new_image->image_path, &error);
			
		if (!new_image->image) {
			g_message ("failed to load image %s", new_image->image_path);
			return FALSE;
		}
		
		if (!i) {
			image_coord = 0; /* First picture at 0 degree */
			new_image->selected = TRUE; /* Is image selected */
		}
		else {
			image_coord = image_coord + ang_step; /* Changing image_coord */
			new_image->selected = FALSE; /* Is image selected */
		}
		
		new_image->image_label = g_strdup(gettext(activities_names[i]));
		
		new_image->xpos = new_image->x_offset = 0;
		
		new_image->ypos = new_image->y_offset = 0;
		
		new_image->width = gdk_pixbuf_get_width (new_image->image);
		
		new_image->height = gdk_pixbuf_get_height (new_image->image);
		
		new_image->alpha_level = 255; /* Alpha level to maximum */
		
		new_image->zoom_ratio = new_image->init_zoom_ratio = 1; /* Zoom ratio full size */
		
		new_image->visible = TRUE; /* Visible by default */
		
		new_image->busy = FALSE; /* Not busy by default */
		
		new_image->angle = image_coord + ANG_OFFSET; /* Initial menu position */
				
		if (new_image->angle > 360)
			new_image->angle = new_image->angle - 360; /* Staying between 0-360 */
		
		new_image->data = NULL;
	
		switch (i) {
				case 0 :
					new_image->image_link = (GtkFunction) activate_divx_from_cd;
					break;;
				case 1 :
					new_image->image_link = (GtkFunction) activate_divx_from_hd;
					break;;
				case 2 :
					new_image->image_link = (GtkFunction) activate_divx_from_nfs;
					break;;
				case 3 :
					new_image->image_link = (GtkFunction) activate_main_menu;
					break;;
		}
		
		new_image->next_image = NULL;
		
		add_image_at_the_end (new_image, &divx_menu_images_list);
	
	}
	
	return TRUE;
}

gboolean
activate_divx_menu (gpointer data)
{
	if (load_divx_menu ()) {
		current_menu_images_list = divx_menu_images_list;
		current_menu = DIVX_MENU;
	}
	
	draw_menu (current_menu_images_list);
	
	return TRUE;
}


gboolean
activate_divx_from_cd (gpointer data)
{
	GDir * media_dir = NULL;
	gchar * file_name = NULL;
	gchar * location = NULL;
	gchar * arg = NULL;
	gchar * command = NULL;
	
	arg = gconf_client_get_string (conf_client, CDROM_PATH, NULL);
	if (!arg) {
		g_warning ("unable to get GConf key for cdrom path");
		arg = "/mnt/cdrom";
	}
	
	command = g_strdup_printf ("/bin/mount %s", arg);
	
	g_spawn_command_line_sync (
			command, NULL, NULL, NULL, NULL);
	
	/* Opening dir */
	
	media_dir = g_dir_open (arg, 0, NULL);
	
	if (!media_dir) { /* Failed to open dir */
		g_message ("unable to open dir %s", arg);
		return;
	}
	
	/* For each file */
	
	while (file_name = (gchar *) g_dir_read_name (media_dir)) {
		if (g_strrstr (file_name,".avi"))
			break;
	}
	
	
	if (!file_name) {
		g_message ("No avi file found");
		g_dir_close (media_dir);
		return FALSE;
	}
	
	g_message ("Found .avi file on cd : %s/%s", arg, file_name);
	
	location = g_strdup_printf ("%s/%s", arg, file_name);
	
	g_dir_close (media_dir);
	
	if (location)
		play_video(location, FROM_CD);
	else
		return FALSE;
	
	g_free (location);	
	
	if (arg) {
		g_free(arg);
		arg = NULL;
	}
	
	return TRUE;
}

gboolean
activate_divx_from_hd (gpointer data)
{
	
	play_video("/tmp/AlienSong.mpg", FROM_HD);
	
	return TRUE;
}

gboolean
activate_divx_from_nfs (gpointer data)
{
	
	play_video("/tmp/AlienSong.mpg", FROM_NFS);
	
	return TRUE;
}

gboolean
restore_divx (gpointer data)
{
	current_status = V_PLAYING;
	
	if (gst_play_get_state (media_play->audio_play) == GST_STATE_PLAYING)
		gst_play_set_state (media_play->audio_play, GST_STATE_PAUSED);
	
	if (!GTK_WIDGET_VISIBLE(media_play->vw))
		gtk_widget_show (GTK_WIDGET (media_play->vw));
	
	/* Giving focus to video window */
		
	GTK_WIDGET_SET_FLAGS (media_play->vw, GTK_CAN_FOCUS);

	gtk_widget_grab_focus ( GTK_WIDGET(media_play->vw) );
	
	if (gst_play_get_state (media_play->video_play) == GST_STATE_PAUSED)
		gst_play_set_state (media_play->video_play, GST_STATE_PLAYING);
		
	/* Hiding our drawing area widget */
		
	gtk_widget_hide(da);
	
	return TRUE;
}

gboolean
eject_cdrom (void)
{
	gchar * arg = NULL;
	gchar * command = NULL;
	
	arg = gconf_client_get_string (conf_client, CDROM_PATH, NULL);
	if (!arg) {
		g_warning ("unable to get GConf key for cdrom path");
		arg = "/mnt/cdrom";
	}
	
	g_message ("Ejecting cdrom (%s)", arg);
	
	command = g_strdup_printf ("umount %s", arg);
	
	g_spawn_command_line_sync (
			command, NULL, NULL, NULL, NULL);
	
	if (command)
		g_free(command);
	
	command = g_strdup_printf ("eject %s", arg);
	
	g_spawn_command_line_sync (
			command, NULL, NULL, NULL, NULL);
	
	if (command)
		g_free(command);
	
	if (arg)
		g_free(arg);
	
	return TRUE;
	
}
