/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* $Id: draw.h,v 1.9 2002/11/12 09:37:52 dolphy Exp $ */

#ifndef DRAW_H
#define DRAW_H

typedef struct im_node { /* Chained list containing images */
	gchar * image_path; /* Path to image file */
	GdkPixbuf * image; /* Pixbuf */
	gchar * image_label; /* Image label */
	gint angle; /* Angle on a circle menu */
	guint xpos; /* X position */
	guint ypos; /* Y position */
	gint x_offset; /* X offset */
	gint y_offset; /* Y offset */
	guint width; /* Image width */
	guint height; /* Image height */
	guint alpha_level; /* Alpha level of image */
	gdouble zoom_ratio; /* Image zoom ratio */
	gdouble init_zoom_ratio; /* Image initial zoom ratio */
	gboolean visible; /* Flag defining if image is visible or not */
	gboolean selected; /* Flag defining if image has been selected */
	gboolean busy; /* Flag defining if menu is busy or not */
	GtkFunction image_link; /* Function called when image is activated */
	gchar * data; /* Additional data that can be passed to above function */
	struct im_node * next_image; /* Pointer to next parameter */
} image_node;

/* Background image and its dimensions */

extern GdkPixbuf * background, * background_orig;

/* Current frame */

extern GdkPixbuf * frame, * frame_backup;

/* Fullscreen setting */

extern gboolean fullscreen;

/* ===========Images list draw============ */

gboolean draw_images (image_node * images_list);

/* ========================================= */

/* ===========Images list tools============= */

/* add_image_at_the_end
	Function adding an image_node at the end of an image_node list
*/

gboolean add_image_at_the_end (image_node * new_image, image_node ** images_list);

/* select_previous
	Function which moves the select flag on the previous node
*/

gboolean select_previous (image_node * images_list);

/* select_next
	Function which moves the select flag on the previous node
*/

gboolean select_next (image_node * images_list);

/* dump_images_list
	Function which will take care of dumping an image_node list
	to stdout
*/

gboolean dump_images_list (image_node * images_list);

/* free_images_list
	Function which will take care of freeing an image_node list
*/

gboolean free_images_list (image_node * images_list);

/* ========================================= */

/* ===========Images list tools============= */

/* wmspec_change_state
	Function that send Xevent to the window manager
	setting fullscreen mode or not
*/

void wmspec_change_state (gboolean add, GdkWindow * window, GdkAtom state1, GdkAtom state2);

/* toggle_fullscreen
	Function which enable or disable fullscreen mode
	according to the fullscreen global flag
*/

gboolean toggle_fullscreen (void);

/* ========================================= */

#endif
