/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* $Id: gst.c,v 1.15 2003/04/12 20:44:30 dolphy Exp $ */

#include "getcontrol.h"

GCMediaPlay * media_play = NULL;

gboolean visualisation = FALSE;

/* Function called when we get the X window created */

gboolean
gst_play_have_xid (GstPlay *play, gint xid)
{	
	g_return_if_fail(GST_IS_PLAY(play));
	
	if (media_play->vw) {
		/* Hiding our drawing area */
		remove_all_timers();
		gtk_widget_hide (da);
		gtk_widget_show (media_play->vw);
		gst_video_widget_set_xembed_xid(
				GST_VIDEO_WIDGET(media_play->vw), xid);
		
	}
		
	return TRUE;
}

/* Function called when end of stream reached */

gboolean
gst_play_eos (GstPlay *play)
{	
	
	g_return_if_fail(GST_IS_PLAY(play));
	
	/* Stopping playback */
	
	gst_play_set_state (play, GST_STATE_READY);
	
	/* Hide video widget from appvbox */
	
	if (GTK_WIDGET_VISIBLE(media_play->vw))
		gtk_widget_hide (GTK_WIDGET(media_play->vw));
		
	gtk_widget_show (da);
	
	gst_play_set_location (play, "/dev/null");
	
	if (play == media_play->audio_play) {
		if (media_play->audio_location) {
			g_free (media_play->audio_location);
			media_play->audio_location = NULL;
		}
		media_play->audio_play_source = UNAVAILABLE;
	}
	if (play == media_play->video_play) {
		if (media_play->video_location) {
			g_free (media_play->video_location);
			media_play->video_location = NULL;
		}
		media_play->video_play_source = UNAVAILABLE;
	}
	
	/* Restoring audio playback */
	
	if ((play == media_play->video_play) &&
		(GST_IS_PLAY(media_play->audio_play)) &&
		(gst_play_get_state (media_play->audio_play) == GST_STATE_PAUSED))
		gst_play_set_state (media_play->audio_play, GST_STATE_PLAYING);
	
	activate_main_menu (NULL);
	
	return TRUE;
}


gboolean
gst_play_have_video_size (GstPlay *play, gint width, gint height, gpointer data)
{
	g_return_if_fail(GST_IS_PLAY(play));
	
	if (media_play->vw)	{
		gst_video_widget_set_source_size(
				GST_VIDEO_WIDGET(media_play->vw),
				width, height);
	}
	
	return TRUE;
}

gboolean
gst_play_got_length (GstPlay *play, gint64 length_nanos)
{
	gint seconds;
	
	g_return_val_if_fail(GST_IS_PLAY(play), FALSE);
	
	if (play == media_play->audio_play) {
		seconds = (gint) (length_nanos / GST_SECOND);
		media_play->audio_length_nanos = length_nanos;
	}
	else if (play == media_play->video_play) {
		seconds = (gint) (length_nanos / GST_SECOND);
		media_play->video_length_nanos = length_nanos;
	}

	return TRUE;
}

gboolean
gst_play_time_tick (GstPlay *play, gint64 time_nanos)
{
	gdouble seek_pos;
	gint seconds;
	gchar * status = NULL;

	g_return_val_if_fail(GST_IS_PLAY(play), FALSE);
	
	if (play == media_play->audio_play) {
		seek_pos = ((gdouble) time_nanos) /
					((gdouble) media_play->audio_length_nanos);
		seconds = (gint) (time_nanos / GST_SECOND);
	
		status = g_strdup_printf ("%s %s | %d/%d",
					_("Playing"),
					media_play->audio_location,
					seconds,
					(gint) (media_play->audio_length_nanos / GST_SECOND));
	}
	else if (play == media_play->video_play) {
		seek_pos = ((gdouble) time_nanos) /
					((gdouble) media_play->video_length_nanos);
		seconds = (gint) (time_nanos / GST_SECOND);
	
		status = g_strdup_printf ("%s %s | %d/%d",
					_("Playing"),
					media_play->video_location,
					seconds,
					(gint) (media_play->video_length_nanos / GST_SECOND));
	}
	
	gnome_appbar_set_status(GNOME_APPBAR (appbar), status);

	if (status)
		g_free(status);
	
	return TRUE;
}

/* destroy_media_play
	Function destroying the media_play object */

gboolean
destroy_media_play (void)
{
		
	if (media_play->vw) {
		gtk_widget_destroy (media_play->vw);
		media_play->vw = NULL;
	}
	
	if (media_play->audio_location) {
		g_free(media_play->audio_location);
		media_play->audio_location = NULL;
	}
	
	if (media_play->video_location) {
		g_free(media_play->video_location);
		media_play->video_location = NULL;
	}
		
	if (media_play->audio_play) {
		g_object_unref ( G_OBJECT(media_play->audio_play) );
		media_play->audio_play = NULL;
	}
	
	if (media_play->video_play) {
		g_object_unref ( G_OBJECT(media_play->video_play) );
		media_play->video_play = NULL;
	}
	
	g_free(media_play);
	
	media_play = NULL;
	
	return TRUE;
	
}

/* media_play_init
	Function creating our media_play object not showing any widget. play_video
	and play_audio will be responsible of showing appropriate stuff */

gboolean
media_play_init (void)
{
	GstElement * audio_audio_sink, * video_sink, * video_audio_sink;
	GstElement * vis_video_sink, * vis_element;
	
	media_play = (GCMediaPlay *) malloc (sizeof (GCMediaPlay));
	
	g_return_val_if_fail (media_play!=NULL, FALSE);
	
	media_play->vw = NULL;
	media_play->audio_location = NULL;
	media_play->video_location = NULL;
	media_play->video_play_source = UNAVAILABLE;
	media_play->audio_play_source = UNAVAILABLE;
	media_play->audio_play = NULL;
	media_play->video_play = NULL;
	media_play->current_play = NULL;
	
	media_play->audio_play = gst_play_new (	GST_PLAY_PIPE_VIDEO_VISUALISATION,
											NULL);
	media_play->video_play = gst_play_new (GST_PLAY_PIPE_VIDEO, NULL);
	
	g_return_val_if_fail (media_play->audio_play!=NULL, FALSE);
	g_return_val_if_fail (media_play->video_play!=NULL, FALSE);
	
	/* Preparing our video widget */
	
	media_play->vw = gst_video_widget_new ();
	
	gst_video_widget_set_cursor_visible (	GST_VIDEO_WIDGET(media_play->vw),
											FALSE);
	gst_video_widget_set_source_size (	GST_VIDEO_WIDGET(media_play->vw),
										100,100);
													
	audio_audio_sink = GST_ELEMENT(gst_gconf_get_default_audio_sink ());
	video_audio_sink = GST_ELEMENT(gst_gconf_get_default_audio_sink ());
	
	g_return_val_if_fail (GST_IS_ELEMENT (audio_audio_sink), FALSE);
	g_return_val_if_fail (GST_IS_ELEMENT (video_audio_sink), FALSE);
	
	/* Putting them in our GstPlay */
	
	gst_play_set_audio_sink (media_play->audio_play, audio_audio_sink);
	gst_play_set_audio_sink (media_play->video_play, video_audio_sink);

	/* Getting default videosink from GConf */
	
	video_sink = GST_ELEMENT(gst_gconf_get_default_video_sink ());
	vis_video_sink = GST_ELEMENT(gst_gconf_get_default_video_sink ());
	
	g_return_val_if_fail (GST_IS_ELEMENT (video_sink), FALSE);
	g_return_val_if_fail (GST_IS_ELEMENT (vis_video_sink), FALSE);
	
	gst_play_set_video_sink (media_play->video_play, video_sink);
	gst_play_set_visualisation_video_sink (	media_play->audio_play,
											vis_video_sink);
	
	/* Getting default visualisation element from GConf */
	
	vis_element = GST_ELEMENT(gst_gconf_get_default_visualisation_element ());
	g_return_val_if_fail (GST_IS_ELEMENT (vis_element), FALSE);
	
	gst_play_set_visualisation_element (media_play->audio_play, vis_element);
	
	/* Putting our video_widget in our app */
	
	gtk_container_add (GTK_CONTAINER(appvbox), media_play->vw);
	
	g_signal_connect (G_OBJECT (media_play->audio_play), "have_vis_xid", 
			G_CALLBACK (gst_play_have_xid), NULL);
	g_signal_connect (G_OBJECT (media_play->audio_play), "time_tick",
			G_CALLBACK (gst_play_time_tick), media_play);
	g_signal_connect (G_OBJECT (media_play->audio_play), "stream_length",
			G_CALLBACK (gst_play_got_length), media_play);
	g_signal_connect (G_OBJECT (media_play->audio_play), "stream_end", 
			G_CALLBACK (gst_play_eos), media_play);	
	g_signal_connect (G_OBJECT (media_play->video_play), "time_tick",
			G_CALLBACK (gst_play_time_tick), media_play);
	g_signal_connect (G_OBJECT (media_play->video_play), "stream_length",
			G_CALLBACK (gst_play_got_length), media_play);
	g_signal_connect (G_OBJECT (media_play->video_play), "stream_end", 
			G_CALLBACK (gst_play_eos), media_play);
	g_signal_connect (G_OBJECT (media_play->video_play), "have_video_size", 
			G_CALLBACK (gst_play_have_video_size), NULL);	
	g_signal_connect (G_OBJECT (media_play->video_play), "have_xid", 
			G_CALLBACK (gst_play_have_xid), NULL);
	
	return TRUE;
	
}

/* play_video
	Function taking care of playing video media at @location.
	If running in fullscreen it creates a gtkpopup and attach events to
	it as xvideosink does not trap events.
	If not running in fullscreen events are trapped by appbar widget giving
	focus to him before.
*/

gboolean
play_video (gchar * location, guint play_source)
{
	GstElement * datasrc = NULL;
	
	g_return_val_if_fail (location!=NULL, FALSE);
	
	if (!media_play)
		media_play_init();
	
	if (media_play->video_location)
		g_free (media_play->video_location);
	
	media_play->video_location = g_strdup (location);
	
	if (g_file_test (location,G_FILE_TEST_EXISTS)) {
		g_message ("location is a local file using filesrc");
		datasrc = gst_element_factory_make ("filesrc", "source");
	}
	else {
		g_message ("location is not a local file using gnomevfssrc");
		datasrc = gst_element_factory_make ("gnomevfssrc", "source");
	}
	
	if (GST_IS_ELEMENT(datasrc))
		gst_play_set_data_src (media_play->video_play, datasrc);
	
	gst_play_set_location (media_play->video_play, media_play->video_location);

	/* Trapping key_press from video window */
		
	g_signal_connect (
				G_OBJECT (media_play->vw),
				"key_press_event",
				(GtkSignalFunc) key_pressed_callback,
				current_menu_images_list);
	
	/* Giving focus to video window */
		
	GTK_WIDGET_SET_FLAGS (media_play->vw, GTK_CAN_FOCUS);

	gtk_widget_grab_focus ( GTK_WIDGET(media_play->vw) );
			
	media_play->video_play_source = play_source;
	
	current_status = V_PLAYING;
	
	/* Autoplay */
	
	if (gst_play_get_state (media_play->audio_play) == GST_STATE_PLAYING)
		gst_play_set_state (media_play->audio_play, GST_STATE_PAUSED);

	gst_play_set_state (media_play->video_play, GST_STATE_PLAYING);
	
	media_play->current_play = media_play->video_play;
	
	return TRUE;
}

/* play_audio
	Function taking care of playing audio media at @location from @play_source.
*/

gboolean
play_audio (gchar * location, guint play_source)
{
	GstElement * datasrc = NULL;
	
	g_return_val_if_fail (location!=NULL, FALSE);
	
	if (!media_play)
		media_play_init();
	
	if (media_play->audio_location)
		g_free (media_play->audio_location);
	
	g_message ("location is %s", location);
	
	media_play->audio_location = g_strdup (location);

	if (g_file_test (location,G_FILE_TEST_EXISTS)) {
		g_message ("location is a local file using filesrc");
		datasrc = gst_element_factory_make ("filesrc", "source");
	}
	else {
		g_message ("location is not a local file using gnomevfssrc");
		datasrc = gst_element_factory_make ("gnomevfssrc", "source");
	}
	
	if (GST_IS_ELEMENT(datasrc))
		gst_play_set_data_src (media_play->audio_play, datasrc);
	
	gst_play_set_location (media_play->audio_play, media_play->audio_location);

	/* Trapping key_press from video window */
		
	g_signal_connect (
				G_OBJECT (media_play->vw),
				"key_press_event",
				(GtkSignalFunc) key_pressed_callback,
				current_menu_images_list);
	
	/* Giving focus to video window */
		
	GTK_WIDGET_SET_FLAGS (media_play->vw, GTK_CAN_FOCUS);

	gtk_widget_grab_focus ( GTK_WIDGET(media_play->vw) );
	
	media_play->audio_play_source = play_source;
	
	/* Visualisation */
	
	if (visualisation)
		gst_play_connect_visualisation (media_play->audio_play, TRUE);
	
	current_status = A_PLAYING;
	
	/* Autoplay */
	
	gst_play_set_state (media_play->audio_play, GST_STATE_PLAYING);
	
	media_play->current_play = media_play->audio_play;
	
	return TRUE;
}

gboolean
toggle_pause_playback (void)
{
	if (!media_play)
		return FALSE;
	
	g_return_val_if_fail(GST_IS_PLAY(media_play->current_play), FALSE);
	
	switch (gst_play_get_state(media_play->current_play)) {
		case GST_STATE_PAUSED :
			gst_play_set_state (media_play->current_play, GST_STATE_PLAYING);
			break;;
		case GST_STATE_PLAYING :
			gst_play_set_state (media_play->current_play, GST_STATE_PAUSED);
			break;;
	}

	return TRUE;
	
}

gboolean
toggle_mute_playback (void)
{
	if (!media_play)
		return FALSE;
	
	g_return_val_if_fail(GST_IS_PLAY(media_play->current_play), FALSE);
	
	if (gst_play_get_mute(media_play->current_play))
		gst_play_set_mute(media_play->current_play, FALSE);
	else
		gst_play_set_mute(media_play->current_play, TRUE);

	return TRUE;
	
}

gboolean
stop_playback (void)
{
	if (!media_play)
		return FALSE;
	
	g_return_val_if_fail(GST_IS_PLAY(media_play->current_play), FALSE);
	
	gst_play_seek_to_time (media_play->current_play, 0);
	gst_play_set_state (media_play->current_play, GST_STATE_PAUSED);
	return TRUE;
	
}

gboolean
start_playback (void)
{
	if (!media_play)
		return FALSE;
	
	g_return_val_if_fail(GST_IS_PLAY(media_play->current_play), FALSE);
	
	gst_play_set_state (media_play->current_play, GST_STATE_PLAYING);
	
	return TRUE;
}

gboolean
set_volume (GstPlay * play,gdouble value)
{
	g_return_val_if_fail(GST_IS_PLAY(play), FALSE);
	
	value = value > 1.0 ? 1.0 : value;
	value = value < 0.0 ? 0.0 : value;
	gst_play_set_volume (play, value);
	return TRUE;
}
