/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/* $Id: interface.c,v 1.21 2003/05/18 16:36:23 dolphy Exp $ */

#include "getcontrol.h"

/* GConf preferences */

GConfClient *conf_client;

/* Our gnome app */

GnomeApp * app = NULL;

/* Widgets */

GtkWidget * appbar = NULL, * da = NULL;
GtkWidget * appvbox = NULL;

/* Menus */

GnomeUIInfo getcontrolmenu[] = {
	{
		GNOME_APP_UI_ITEM, N_("_Fullscreen"),
		NULL,
		(gpointer) toggle_fullscreen, NULL, NULL,
		GNOME_APP_PIXMAP_STOCK, "gtk-zoom-fit",
		0, (GdkModifierType) 0, NULL
	},
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_QUIT_ITEM(getcontrol_quit_callback, NULL),
	GNOMEUIINFO_END
};

GnomeUIInfo settingsmenu[] = {
	GNOMEUIINFO_MENU_PREFERENCES_ITEM(getcontrol_about_callback, NULL),
	GNOMEUIINFO_END
};

GnomeUIInfo helpmenu[] = {
	GNOMEUIINFO_HELP("getcontrol"),
	GNOMEUIINFO_MENU_ABOUT_ITEM(getcontrol_about_callback, NULL),
	GNOMEUIINFO_END
};

GnomeUIInfo mainmenu[] = { 
	{
		GNOME_APP_UI_SUBTREE, N_("Getcontrol"),
		NULL,
		getcontrolmenu, NULL, NULL,
		GNOME_APP_PIXMAP_NONE, NULL,
		0, (GdkModifierType) 0, NULL
	},
	GNOMEUIINFO_MENU_SETTINGS_TREE(settingsmenu),
	GNOMEUIINFO_MENU_HELP_TREE(helpmenu),
	GNOMEUIINFO_END
};


/* Main */

int
main (int argc, char **argv)
{
	
	/* Defining additional params from command line */
	
	static const struct poptOption options[] = {
		{ NULL, '\0', 0, NULL, 0 }
	}; 

	GnomeClient * client;
	GdkBitmap *empty_bitmap;
	GdkColor useless;
	GdkCursor *cursor;
	char invisible_cursor_bits[] = { 0x0 }; 

	/* Gettext management */

#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif
	
	/* Initing */
	
	gdk_rgb_init ();
	
	gst_init (&argc, &argv);
	
	/* Using opt as default scheduler */
	gst_scheduler_factory_set_default_name ("opt");
	
	gnome_program_init (
				"Getcontrol", VERSION,
				LIBGNOMEUI_MODULE,
				argc, argv,
				GNOME_PARAM_POPT_TABLE, options,
				GNOME_PARAM_APP_DATADIR, DATADIR, NULL);

	/* Get the default GConfClient */

	conf_client = gconf_client_get_default ();
	
	if (!load_config()) {
		g_message ("main(): Failed to load config params from GConf");
		exit (EXIT_FAILURE);
	}
	if (!load_background()) {
		g_message ("main(): Failed to load background");
		exit (EXIT_FAILURE);
	}
	if (!load_main_menu()) {
		g_message ("main(): Failed to load main menu");
		exit (EXIT_FAILURE);
	}
	
	/* Current menu is the main menu at startup */

	current_menu_images_list = main_menu_images_list;
	
	current_menu = MAIN_MENU;
	
	client= gnome_master_client ();

	/* Setting icon */
	
	gnome_window_icon_set_default_from_file (DATADIR"/getcontrol/getcontrol.png");

	/* Creating app */
	
	app = GNOME_APP(gnome_app_new("getcontrol", _("Getcontrol")));

	gtk_widget_set_usize (GTK_WIDGET(app), back_width, back_height);
	
	gtk_window_set_resizable (GTK_WINDOW(app), TRUE);

	appbar = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_USER);
	
	gnome_app_set_statusbar(GNOME_APP (app), GTK_WIDGET(appbar));

	gnome_appbar_set_status(GNOME_APPBAR (appbar), _("Getcontrol is now running"));

	gnome_app_create_menus(GNOME_APP(app), mainmenu);

	gnome_app_install_menu_hints(GNOME_APP (app), mainmenu);

	/* Connecting signals */
	
	g_signal_connect (
			G_OBJECT (app), "delete_event",
			G_CALLBACK(getcontrol_quit_callback), NULL);

	gtk_widget_push_colormap (gdk_rgb_get_colormap ());

	/* Creating our drawing area */

	da = gtk_drawing_area_new ();
	
	GTK_WIDGET_SET_FLAGS (da, GTK_CAN_FOCUS);
	
	gtk_widget_pop_colormap ();

	/* Connecting expose and button_press_event */

	g_signal_connect (
					GTK_OBJECT (da), "expose_event",
					(GtkSignalFunc) expose_callback, NULL);

	g_signal_connect (
					GTK_OBJECT (da), "key_press_event",
					(GtkSignalFunc) key_pressed_callback,
					current_menu_images_list);
					
	g_signal_connect (
					GTK_OBJECT (da), "configure_event",
					(GtkSignalFunc) configure_callback, NULL);

	/* Setting events we would like to receive */

	gtk_widget_set_events (da,
				GDK_EXPOSURE_MASK
				| GDK_LEAVE_NOTIFY_MASK
				| GDK_BUTTON_PRESS_MASK
				| GDK_KEY_PRESS_MASK
				| GDK_KEY_RELEASE_MASK
				| GDK_POINTER_MOTION_MASK
				| GDK_POINTER_MOTION_HINT_MASK);
			 
	gtk_widget_grab_focus ( GTK_WIDGET( da) );
	
	/* Packing in a vbox */
	
	appvbox = gtk_vbox_new (TRUE, 0);
	
	gtk_container_add (GTK_CONTAINER(appvbox), da);
	
	/* Putting our widget in the Gnome app */
	
	gnome_app_set_contents (GNOME_APP (app), appvbox);
	
	/* Showing widgets */

	gtk_widget_show_all (GTK_WIDGET(app));

	/* Removing cursor */

	useless.red = useless.green = useless.blue = useless.pixel = 0;

	empty_bitmap = gdk_bitmap_create_from_data (	da->window,
													invisible_cursor_bits,
													1, 1);

	cursor = gdk_cursor_new_from_pixmap (	empty_bitmap,
											empty_bitmap,
											&useless,
											&useless, 0, 0);
		
	gdk_window_set_cursor (da->window, cursor);
	
	gdk_cursor_unref (cursor);

	g_object_unref (empty_bitmap);
	
	/* Telling GConf to notify us at any change on our path */
	
	gconf_client_add_dir (conf_client,
			      "/apps/getcontrol",
			      GCONF_CLIENT_PRELOAD_NONE,
			      NULL);

	/* Listen to changes to our global keys. */
	
	gconf_client_notify_add (conf_client,
				 "/apps/getcontrol/global",
				 gconf_notify_global,
				 NULL,
				 NULL,
				 NULL);
	
	/* Enabling fullscreen or not */
	
	toggle_fullscreen ();
	
#ifdef HAVE_LIRC
	
	if ( !my_lirc_init() )
		gnome_error_dialog_parented(_("Could not start lirc"), GTK_WINDOW(app));
	else
		start_lirc();
	
#endif
	
	/* Starting GTK loop */

	gtk_main ();

	return 0;
}
