/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/* $Id: lirc.c,v 1.9 2003/04/12 20:44:30 dolphy Exp $ */

#include "getcontrol.h"

#ifdef HAVE_LIRC

struct lirc_config * remote_config = NULL;

gint lirc_fd = -1, input_tag = 0;

/* execute_lirc_command
	Function analyzing the lirc order and executing corresponding order
*/

gboolean
execute_lirc_command (gchar *cmd)
{
	image_node * l_images_list = NULL;
	image_node * new_image = NULL;
	
	if (g_strcasecmp (cmd, LIRC_LEFT) == 0) {
		g_message ("lirc left");
		
		switch (current_status) {
			case IDLE :
			case SELECTED :
				remove_all_timers ();
				direction = LEFT;
				menu_rotation_manager (current_menu_images_list);
				return TRUE;
			case SCREEN_SAVING :
				remove_all_timers ();
				draw_menu (current_menu_images_list);
				return TRUE;
			case ROTATING :
				stack_order (&order_queue, LEFT, ROTATE_ORDER);
				return TRUE;
			default :
				return TRUE;
		}
			
	}
	else if (g_strcasecmp (cmd, LIRC_RIGHT) == 0) {
		g_message ("lirc right");
		
		switch (current_status) {
			case IDLE :
			case SELECTED :
				remove_all_timers ();
				direction = RIGHT;
				menu_rotation_manager (current_menu_images_list);
				return TRUE;
			case SCREEN_SAVING :
				remove_all_timers ();
				draw_menu (current_menu_images_list);
				return TRUE;
			case ROTATING :
				stack_order (&order_queue, RIGHT, ROTATE_ORDER);
				return TRUE;
			default :
				return TRUE;
		}
	}
	else if (g_strcasecmp (cmd, LIRC_UP) == 0) {
		g_message ("lirc up");
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_DOWN) == 0) {
		g_message ("lirc down");
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_FULLSCREEN) == 0) {
		g_message ("lirc fullscreen");
		
		if (fullscreen)
			fullscreen = FALSE;
		else
			fullscreen = TRUE;
					
		/* Storing the change in GConf */
	
		gconf_client_set_bool (conf_client,
					FULLSCREEN_PATH, fullscreen, NULL);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_VALIDATE) == 0) {
		g_message ("lirc validate");
		switch (current_status) {
			case IDLE :
			case SELECTED :
				remove_all_timers ();
				l_images_list = current_menu_images_list;
				while ( !(l_images_list->selected) && (l_images_list) )
					l_images_list = l_images_list->next_image;
				l_images_list->image_link (l_images_list->data);
				return TRUE;
			case SCREEN_SAVING :
				remove_all_timers ();
				draw_menu (current_menu_images_list);
				return TRUE;
			case ROTATING :
				stack_order (&order_queue, LEFT, ACTIVATE_ORDER);
				return TRUE;
			default :
				return TRUE;
		}
		
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_MAIN_MENU) == 0) {
		g_message ("lirc main menu");
		
		switch (current_status) {
			case V_PLAYING :
				gst_play_set_state (	media_play->video_play,
										GST_STATE_PAUSED);
				if (GTK_WIDGET_VISIBLE(media_play->vw))
					gtk_widget_hide (GTK_WIDGET (media_play->vw));
					
				gtk_widget_show (da);
					
				gtk_widget_grab_focus ( GTK_WIDGET( da) );
				
				/* Restoring Audio playback if paused */
					
				if ((GST_IS_PLAY(media_play->audio_play)) &&
					(	gst_play_get_state (media_play->audio_play) ==
						GST_STATE_PAUSED ))
				{
					gst_play_set_state (	media_play->audio_play,
											GST_STATE_PLAYING);
					media_play->current_play = media_play->audio_play;
				}
					
				activate_main_menu (main_menu_images_list);
				break;;
			case A_PLAYING :
				if (GTK_WIDGET_VISIBLE(media_play->vw))
					gtk_widget_hide (GTK_WIDGET (media_play->vw));
					
				gtk_widget_show (da);
					
				gtk_widget_grab_focus ( GTK_WIDGET( da) );
			default :
				g_message ("main menu");
				activate_main_menu (main_menu_images_list);
		}
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_PLAY) == 0) {
		g_message ("lirc play");
		start_playback();
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_PAUSE) == 0) {
		g_message ("lirc pause");
		toggle_pause_playback ();
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_STOP) == 0) {
		g_message ("lirc stop");
		switch (current_status) {
			case V_PLAYING :
				if ((	gst_play_get_state (media_play->video_play) ==
						GST_STATE_PLAYING) ||
					(	gst_play_get_state (media_play->video_play) ==
						GST_STATE_PAUSED))
					gst_play_eos (media_play->video_play);
				break;;
			default :
				if ((	gst_play_get_state (media_play->audio_play) ==
						GST_STATE_PLAYING) ||
					(	gst_play_get_state (media_play->audio_play) ==
						GST_STATE_PAUSED))
					gst_play_eos (media_play->audio_play);
				break;;
			}
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_MUTE) == 0) {
		g_message ("lirc mute");
		toggle_mute_playback ();
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_EJECT) == 0) {
		g_message ("lirc eject");
	
		remove_all_timers();
		
		new_image = (image_node *) malloc (sizeof (image_node));
		
		new_image->image_path = g_strdup_printf (
										"%s"G_DIR_SEPARATOR_S"eject.png",
										current_theme);
		
		new_image->image = gdk_pixbuf_new_from_file (new_image->image_path, NULL);
		
		new_image->selected = FALSE; /* Is image selected */
		
		new_image->image_label = g_strdup("Eject");
		
		new_image->width = gdk_pixbuf_get_width (new_image->image);
		
		new_image->height = gdk_pixbuf_get_height (new_image->image);
		
		new_image->xpos = xmid - (new_image->width / 2);
		
		new_image->x_offset = 0;
		
		new_image->ypos = ymid - (new_image->height / 2);
			
		new_image->y_offset = 0;
		
		new_image->alpha_level = 255; /* Alpha level to maximum */
		
		new_image->zoom_ratio = new_image->init_zoom_ratio = 1; /* Zoom ratio full size */
		
		new_image->visible = TRUE; /* Visible by default */
			
		new_image->busy = FALSE; /* Not busy by default */
			
		new_image->data = NULL;
			
		new_image->next_image = NULL;
		
		add_image_at_the_end (new_image, &current_menu_images_list);
			
		draw_images (current_menu_images_list);
			
		sleep (1);
			
		if (	(media_play) &&
				(media_play->video_play_source == FROM_CD)) {
			gst_play_eos (media_play->video_play);
		}
		else if (	(media_play) &&
				(media_play->audio_play_source == FROM_CD)) {
			gst_play_eos (media_play->audio_play);
		}
		else {
			remove_last_image (&current_menu_images_list);
			draw_images (current_menu_images_list);
		}
		eject_cdrom ();
			
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_ZERO) == 0) {
		if (	(current_status == V_PLAYING) &&
				(GST_IS_PLAY(media_play->video_play)) )
			gst_play_seek_to_time (media_play->video_play, 0);
		else if (GST_IS_PLAY (media_play->audio_play))
			gst_play_seek_to_time (media_play->audio_play, 0);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_ONE) == 0) {
		if (	(current_status == V_PLAYING) &&
				(GST_IS_PLAY(media_play->video_play)) )
			gst_play_seek_to_time (media_play->video_play,
				(10LL * media_play->video_length_nanos) / 100LL);
		else if (GST_IS_PLAY (media_play->audio_play))
			gst_play_seek_to_time (media_play->audio_play,
				(10LL * media_play->audio_length_nanos) / 100LL);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_TWO) == 0) {
		if (	(current_status == V_PLAYING) &&
				(GST_IS_PLAY(media_play->video_play)) )
			gst_play_seek_to_time (media_play->video_play,
				(20LL * media_play->video_length_nanos) / 100LL);
		else if (GST_IS_PLAY (media_play->audio_play))
			gst_play_seek_to_time (media_play->audio_play,
				(20LL * media_play->audio_length_nanos) / 100LL);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_THREE) == 0) {
		if (	(current_status == V_PLAYING) &&
				(GST_IS_PLAY(media_play->video_play)) )
			gst_play_seek_to_time (media_play->video_play,
				(30LL * media_play->video_length_nanos) / 100LL);
		else if (GST_IS_PLAY (media_play->audio_play))
			gst_play_seek_to_time (media_play->audio_play,
				(30LL * media_play->audio_length_nanos) / 100LL);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_FOUR) == 0) {
		if (	(current_status == V_PLAYING) &&
				(GST_IS_PLAY(media_play->video_play)) )
			gst_play_seek_to_time (media_play->video_play,
				(40LL * media_play->video_length_nanos) / 100LL);
		else if (GST_IS_PLAY (media_play->audio_play))
			gst_play_seek_to_time (media_play->audio_play,
				(40LL * media_play->audio_length_nanos) / 100LL);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_FIVE) == 0) {
		if (	(current_status == V_PLAYING) &&
				(GST_IS_PLAY(media_play->video_play)) )
			gst_play_seek_to_time (media_play->video_play,
				(50LL * media_play->video_length_nanos) / 100LL);
		else if (GST_IS_PLAY (media_play->audio_play))
			gst_play_seek_to_time (media_play->audio_play,
				(50LL * media_play->audio_length_nanos) / 100LL);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_SIX) == 0) {
		if (	(current_status == V_PLAYING) &&
				(GST_IS_PLAY(media_play->video_play)) )
			gst_play_seek_to_time (media_play->video_play,
				(60LL * media_play->video_length_nanos) / 100LL);
		else if (GST_IS_PLAY (media_play->audio_play))
			gst_play_seek_to_time (media_play->audio_play,
				(60LL * media_play->audio_length_nanos) / 100LL);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_SEVEN) == 0) {
		if (	(current_status == V_PLAYING) &&
				(GST_IS_PLAY(media_play->video_play)) )
			gst_play_seek_to_time (media_play->video_play,
				(70LL * media_play->video_length_nanos) / 100LL);
		else if (GST_IS_PLAY (media_play->audio_play))
			gst_play_seek_to_time (media_play->audio_play,
				(70LL * media_play->audio_length_nanos) / 100LL);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_EIGHT) == 0) {
		if (	(current_status == V_PLAYING) &&
				(GST_IS_PLAY(media_play->video_play)) )
			gst_play_seek_to_time (media_play->video_play,
				(80LL * media_play->video_length_nanos) / 100LL);
		else if (GST_IS_PLAY (media_play->audio_play))
			gst_play_seek_to_time (media_play->audio_play,
				(80LL * media_play->audio_length_nanos) / 100LL);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_NINE) == 0) {
		if (	(current_status == V_PLAYING) &&
				(GST_IS_PLAY(media_play->video_play)) )
			gst_play_seek_to_time (media_play->video_play,
				(90LL * media_play->video_length_nanos) / 100LL);
		else if (GST_IS_PLAY (media_play->audio_play))
			gst_play_seek_to_time (media_play->audio_play,
				(90LL * media_play->audio_length_nanos) / 100LL);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_VOLUP) == 0) {
		if (	(current_status == V_PLAYING) &&
				(GST_IS_PLAY(media_play->video_play)) )
			set_volume (media_play->video_play, 
				gst_play_get_volume (media_play->video_play) + 0.1);
		if ( GST_IS_PLAY(media_play->audio_play) )
			set_volume (media_play->audio_play, 
				gst_play_get_volume (media_play->audio_play) + 0.1);
		return TRUE;
	}
	else if (g_strcasecmp (cmd, LIRC_VOLDOWN) == 0) {
		if (	(current_status == V_PLAYING) &&
				(GST_IS_PLAY(media_play->video_play)) )
			set_volume (media_play->video_play, 
				gst_play_get_volume (media_play->video_play) - 0.1);
		if ( GST_IS_PLAY(media_play->audio_play) )
			set_volume (media_play->audio_play, 
				gst_play_get_volume (media_play->audio_play) - 0.1);
		return TRUE;
	}
	else
		g_message ("unrecognized lirccmd: %s\n", cmd);
	
	return TRUE;
}

gboolean
my_lirc_init (void)
{
	g_message ("Trying to bring up lirc");
	
	if ( (lirc_fd = lirc_init ("getcontrol", 0)) <= 0 ) {
		g_message ("lirc_init");
		return FALSE;
	}

	if (lirc_readconfig (NULL, &remote_config, NULL) != 0) {
		g_message ("lirc_readconfig");
		lirc_deinit ();
		lirc_fd = -1;
		return FALSE;
	}

	fcntl (lirc_fd, F_SETFL, O_NONBLOCK);
	fcntl (lirc_fd, F_SETFD, FD_CLOEXEC);

	return TRUE;
}

gboolean
my_lirc_deinit(void)
{
	
	if (lirc_fd <= 0)
		return FALSE;
	
	g_message("Shutting down lirc");
	
	gdk_input_remove (input_tag);
	
	lirc_freeconfig(remote_config);
	lirc_deinit ();
	
	input_tag = 0;
	
	return TRUE;
}

void
lirc_has_data_cb (gpointer data, gint fd, GdkInputCondition condition)
{
	gchar * code, * cmd;
	gint ret = -1;

	while (lirc_nextcode (&code) == 0 && code != NULL) {
		ret = 0;
		while (lirc_code2char (remote_config, code, &cmd) == 0 && cmd != NULL) {
			g_message ("lirc command: %s\n", cmd);
			execute_lirc_command(cmd);
		}
		if (code)
			g_free (code);
	}

	if (ret == -1) {
		
		g_message("An lirc error occured");
		gdk_input_remove (input_tag);
		lirc_freeconfig (remote_config);
		remote_config = NULL;
		lirc_deinit ();
	}
}

gboolean
start_lirc(void)
{

	if (input_tag)
		return FALSE;

	if (lirc_fd<0)
		return FALSE;

	input_tag = gdk_input_add (lirc_fd, GDK_INPUT_READ, lirc_has_data_cb, NULL);
	
	return TRUE;
}

#endif
