/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* $Id: menu.c,v 1.32 2003/04/12 20:44:30 dolphy Exp $ */
 
#include "getcontrol.h"

/* ==========Order Queue====== */

order_node * order_queue = NULL;

/* =========================== */

/* ===========Timers========== */

/* Delay between frames when animating */

guint animation_frame_delay = 25;

/* Delay before saving screen */

guint screensaver_delay = 10000;

/* Delay before selecting menu */ 

guint selection_delay = 1000;

/* Actual iteration of busy animation */

guint busy_iteration = 0;

/* Global timer id */

guint select_timeout_id = 0, screen_saver_timeout_id = 0;

/* Animation timers */

guint ss_draw_timeout_id = 0, draw_timeout_id = 0;
guint busy_draw_timeout_id = 0, ss_bg_switch_timeout_id = 0;

guint bg_switch_delay = 0;

/* Reloading timer flag */

gboolean reload_timers = FALSE;

/* =========================== */

gboolean direction = LEFT;

gint back_width, back_height, xmid, ymid, radius, draw_width, draw_height;

/* Number of degrees remaining to rotate */

gint left_to_move, total_move;

/* Global info on what we are doing */

gint current_status = IDLE;

/* Global info on which menu we are */

guint current_menu = MAIN_MENU;

/* Current screensaver background id */

guint ss_bg_id = 1;

/* Global info on the current theme */

gchar * current_theme = NULL;

/* Current theme font markup */

gchar * font_markup = NULL;

/* Label of the selected menu */

gchar * selected_label = NULL;

/* ============Menus============ */

image_node * main_menu_images_list = NULL, * prefs_menu_images_list = NULL;
image_node * themes_menu_images_list = NULL, * screensaver_menu_images_list = NULL;
image_node * anim_menu_images_list = NULL, * current_menu_images_list = NULL;

GSList * ss_bg_list = NULL;

/* ==================Loading functions================== */

/* load_timers
	Function which loads timers information from Gconf
*/

gboolean
load_config (void)
{
	gchar * old_theme = NULL;
	gchar * theme_config_file = NULL;
	xmlconfig * theme_config = NULL;
	gchar * font_color = NULL, * font_size = NULL;
	gchar * font_name = NULL;

	/* Loading timer informations */
	
	animation_frame_delay = gconf_client_get_int (
								conf_client,
								ANIM_DELAY_PATH, NULL);

	if (!animation_frame_delay)
		animation_frame_delay = 25;
	
	selection_delay = gconf_client_get_int (
						conf_client,
						SELECT_DELAY_PATH, NULL);

	if (!selection_delay)
		selection_delay = 2000;
	
	screensaver_delay = gconf_client_get_int (
							conf_client,
							SS_DELAY_PATH, NULL);

	if (!screensaver_delay)
		screensaver_delay = 60000;
	
	bg_switch_delay = gconf_client_get_int (
							conf_client,
							BG_SWITCH_DELAY_PATH, NULL);
	if (!bg_switch_delay)
		bg_switch_delay = 25000;
	
	reload_timers = TRUE;
	
	if (select_timeout_id) {
		gtk_timeout_remove (select_timeout_id);
		select_timeout_id = gtk_timeout_add (	selection_delay,
													menu_selection_manager,
													current_menu_images_list);
	}
	
	if ( (screen_saver_timeout_id) && (screensaver_delay) ) {
		gtk_timeout_remove (screen_saver_timeout_id);
		screen_saver_timeout_id = gtk_timeout_add (
									screensaver_delay + selection_delay,
									screen_saver_manager,
									current_menu_images_list);
	}
		
	/* Loading theme */
	
	if (!current_theme) {
		
		current_theme = gconf_client_get_string (conf_client, THEME_PATH, NULL);
		
		/* Checking if the theme stored in gconf is valid
			if not using default builtin theme */
		
		if (!check_theme_validity (current_theme)) { /* Using default theme */
			g_free (current_theme);
			current_theme = g_build_path ( G_DIR_SEPARATOR_S,
								DATADIR,
								"getcontrol",
								"themes",
								"default", NULL);
			g_message ("Using default theme in %s", current_theme);
			if (!check_theme_validity (current_theme)) {
				g_message("Even theme in %s is invalid !!", current_theme);
				return FALSE;
			}
		}
		
		theme_config_file = g_build_path ( G_DIR_SEPARATOR_S,
								current_theme,
								"theme.xml", NULL);
		
		theme_config = xmlconfig_open (theme_config_file);
		
		if (theme_config_file)
			g_free (theme_config_file);
		
		if (font_markup)
			g_free (font_markup);
		
		font_color= xmlconfig_get_content (theme_config,
						"/theme/font/font_color");
		if (!font_color)
			font_color = g_strdup ("black");
		
		font_size = xmlconfig_get_content (theme_config,
						"/theme/font/font_size");
		if (!font_size)
			font_size = g_strdup ("20420");
		
		font_name = xmlconfig_get_content (theme_config,
						"/theme/font/font_name");
		if (!font_name)
			font_name = g_strdup ("Sans");
		
		font_markup = g_strdup_printf (
						"<span foreground='%s' size='%s' font_desc='%s'>",
						font_color, font_size, font_name);
		
		if (font_color) g_free (font_color);
		if (font_size) g_free (font_size);
		if (font_name) g_free (font_name);
		
		if (theme_config)
			xmlconfig_close (theme_config);
	}
	else {
		old_theme = current_theme;
	
		current_theme = gconf_client_get_string (conf_client, THEME_PATH, NULL);
	
		/* If Theme from gconf is different from actual one and is valid 
			then we load it */
		
		if (
			(g_strcasecmp (current_theme, old_theme) != 0) && 
			(check_theme_validity(current_theme))
		) {
					
			theme_config_file = g_build_path ( G_DIR_SEPARATOR_S,
									current_theme,
									"theme.xml", NULL);
		
			theme_config = xmlconfig_open (theme_config_file);
		
			if (theme_config_file)
				g_free (theme_config_file);
		
			if (font_markup)
				g_free (font_markup);
			
			font_color= xmlconfig_get_content (theme_config,
						"/theme/font/font_color");
			if (!font_color)
				font_color = g_strdup ("black");
		
			font_size = xmlconfig_get_content (theme_config,
						"/theme/font/font_size");
			if (!font_size)
				font_size = g_strdup ("20420");
		
			font_name = xmlconfig_get_content (theme_config,
						"/theme/font/font_name");
			if (!font_name)
				font_name = g_strdup ("Sans");
		
			font_markup = g_strdup_printf (
						"<span foreground='%s' size='%s' font_desc='%s'>",
						font_color, font_size, font_name);
		
			if (font_color) g_free (font_color);
			if (font_size) g_free (font_size);
			if (font_name) g_free (font_name);
			
			if (theme_config)
				xmlconfig_close (theme_config);
			
			/* Theme setting has changed */
			
			if (background_orig)
				if (load_background ()) {
					
					/* If background loading for this theme fails we stop */
		
					if (main_menu_images_list)
						load_main_menu ();
		
					if (prefs_menu_images_list)
						load_prefs_menu ();
		
					if (themes_menu_images_list)
						load_themes_menu ();
		
					if (screensaver_menu_images_list)
						load_screensaver_menu ();
		
					if (anim_menu_images_list)
						load_anim_prefs_menu ();
					
					switch (current_menu) {
						case MAIN_MENU :
							current_menu_images_list = main_menu_images_list;
							break;;
						case PREFS_MENU :
							current_menu_images_list = prefs_menu_images_list;
							break;;
						case THEMES_MENU :
							current_menu_images_list = themes_menu_images_list;
							break;;
						case ANIM_MENU :
							current_menu_images_list = anim_menu_images_list;
							break;;
						case SS_MENU :
							current_menu_images_list = screensaver_menu_images_list;
							break;;
					}
					
					draw_menu (current_menu_images_list);
				}
				
			if (old_theme) {
				g_free (old_theme);
				old_theme = NULL;
			}
		}
		else {
					
			/* Reverting to previous theme */
					
			g_message ("Invalid theme %s reverting to %s",
									current_theme, old_theme);
					
			current_theme = old_theme;
					
			load_background ();
		}
	
	}
	
	visualisation = gconf_client_get_bool (conf_client, VIS_PATH, NULL);
	
	toggle_visualisation ();
	
	fullscreen = gconf_client_get_bool (conf_client, FULLSCREEN_PATH, NULL);
		
	if (da) /*  If drawing area is already
				allocated (not the first config load)*/
		toggle_fullscreen ();
	
	return TRUE;
	
}

/* load_background
	Function which loads the background pictures and draw it scaled in the
	background buffer for next draw event */

gboolean
load_background (void) {
	
	gchar * background_path = NULL;
	GError * error = NULL;
	gdouble width_ratio_scale, height_ratio_scale;
	
	remove_all_timers ();
	
	/* building background filename */
	
	if (background_path)
		g_free(background_path);
	
	background_path = g_build_path ( G_DIR_SEPARATOR_S,
						current_theme,
						"background.jpg", NULL);
	
	/* Freeing previous background */
	
	if (background_orig)
		gdk_pixbuf_unref (background_orig);
	
	/* Loading the JPEG background image */

	background_orig = gdk_pixbuf_new_from_file ( background_path, &error);
	
	if (!background_orig) {
		g_message ("Failed to load background at %s", background_path);
		return FALSE;
	}
	
	/* Freeing the name's string */

	if (background_path)
		g_free(background_path);
	
	/* Getting background size */

	back_width = gdk_pixbuf_get_width (background_orig);
	back_height = gdk_pixbuf_get_height (background_orig);
	
	/* If we are called before the first application's configure event
	the background buffer has not been created. If it exists that means
	that drawing area width & height have been calculated and we need
	to redraw in this buffer scaling the background_orig */
	
	if (background) { /* Has the background buffer been created already */
		
		width_ratio_scale = (gdouble) draw_width / (gdouble) back_width;
		height_ratio_scale = (gdouble) draw_height / (gdouble) back_height;
	
		/* Drawing & scaling our background */
	
		gdk_pixbuf_composite (
			background_orig,
			background,
			0, 0,
			draw_width, draw_height,
			0, 0,
			width_ratio_scale, height_ratio_scale,
			GDK_INTERP_BILINEAR,
			255);
		
	}
	
	return TRUE;
}

/* load_main_menu
	Function which loads the main menu's pictures in an image list memory
	structure */

gboolean
load_main_menu (void) {
	gint i, image_coord = 0, ang_step;
	gint nb_activities;
	GError * error = NULL;
	image_node * new_image;
	
	gchar * activities_names[] = {
		gettext_noop ("divx"),
		gettext_noop ("dvd"),
		gettext_noop ("audio"),
		gettext_noop ("mame"),
		gettext_noop ("photos"),
		gettext_noop ("prefs")
	};
	
	guint enabled_activities = 0;
	
	remove_all_timers ();	
	
	if (main_menu_images_list) { /* Freeing already loaded pics */
		free_images_list (main_menu_images_list);
		main_menu_images_list = NULL;
	}
	
	
	/* How many activities do we have */
	
	nb_activities = sizeof (activities_names) / sizeof (activities_names[0]);
	
	for (i = 0; i < nb_activities; i++) {
		gchar * path_to_flag;
		
		path_to_flag = g_strdup_printf ("/apps/getcontrol/modules/%s", activities_names[i]);
		
		if (gconf_client_get_bool(conf_client, path_to_flag, NULL))
			enabled_activities++;
		
		if (path_to_flag)
			g_free (path_to_flag);
		
	}
		
	if (!enabled_activities) {
		g_message ("Hmmm all activities are disabled...");
		exit (1);
	}
	
	ang_step = 360 / enabled_activities;
	
	for (i = 0; i < nb_activities; i++) {
		gchar * path_to_flag;
		
		path_to_flag = g_strdup_printf ("/apps/getcontrol/modules/%s", activities_names[i]);
		
		if (gconf_client_get_bool(conf_client, path_to_flag, NULL)) {
			
			new_image = (image_node *) malloc (sizeof(image_node));
			
			new_image->data = NULL;
			
			switch (i) {
				case 0 :
					if ((media_play) &&
						(	gst_play_get_state(media_play->video_play) ==
							GST_STATE_PAUSED) ) {
						new_image->image_link = (GtkFunction) restore_divx;
						new_image->busy = TRUE;
					}
					else {
						new_image->image_link = (GtkFunction) activate_divx_menu;
						new_image->busy = FALSE;
					}
					break;;
				case 1 :
					new_image->image_link = (GtkFunction) activate_dummy_menu;
					new_image->busy = FALSE;
					break;;
				case 2 :
					if ((media_play) &&
						(	gst_play_get_state(media_play->audio_play) !=
							GST_STATE_READY)	)
						new_image->busy = TRUE;
					else
						new_image->busy = FALSE;
					new_image->image_link = (GtkFunction) activate_audio_menu;
					break;;
				case 3 :
					new_image->image_link = (GtkFunction) activate_dummy_menu;
					new_image->busy = FALSE;
					break;;
				case 4 :
					new_image->image_link = (GtkFunction) activate_dummy_menu;
					new_image->busy = FALSE;
					break;;
				case 5 :
					new_image->image_link = (GtkFunction) activate_prefs_menu;
					new_image->busy = FALSE;
					break;;
			}
			
			new_image->image_path = g_strdup_printf (
									"%s"G_DIR_SEPARATOR_S"%s.png",
									current_theme,
									activities_names[i]);
			
			new_image->image = gdk_pixbuf_new_from_file (new_image->image_path, &error);
			
			if (!new_image->image) {
				g_message ("failed to load image %s", new_image->image_path);
				return FALSE;
			}
		
			if (!i) {
				image_coord = 0; /* First picture at 0 degree */
				new_image->selected = TRUE; /* Is image selected */
			}
			else {
				image_coord = image_coord + ang_step; /* Changing image_coord */
				new_image->selected = FALSE; /* Is image selected */
			}
		
			new_image->image_label = g_strdup(gettext(activities_names[i]));
			
			new_image->xpos = new_image->x_offset = 0;
		
			new_image->ypos = new_image->y_offset = 0;
		
			new_image->width = gdk_pixbuf_get_width (new_image->image);
		
			new_image->height = gdk_pixbuf_get_height (new_image->image);
		
			new_image->alpha_level = 255; /* Alpha level to maximum */
		
			new_image->zoom_ratio = new_image->init_zoom_ratio = 1; /* Zoom ratio full size */
		
			new_image->visible = TRUE; /* Visible by default */
		
			new_image->angle = image_coord + ANG_OFFSET; /* Initial menu position */
				
			if (new_image->angle > 360)
				new_image->angle = new_image->angle - 360; /* Staying between 0-360 */
			
			new_image->next_image = NULL;
		
			add_image_at_the_end (new_image, &main_menu_images_list);
	
		}
		
		if (path_to_flag)
			g_free (path_to_flag);
	}
		
	return TRUE;
}

/* load_prefs_menu
	Function which loads the preferences menu's pictures in an image list
	memory structure */

gboolean
load_prefs_menu (void) {
	gint i, image_coord = 0, ang_step;
	gint nb_activities;
	GError * error = NULL;
	image_node * new_image;
	
	gchar * activities_names[] = {
		gettext_noop ("prefs_themes"),
		gettext_noop ("prefs_anim"),
		gettext_noop ("prefs_screensaver"),
		gettext_noop ("back")
	};
	
	remove_all_timers ();
	
	if (prefs_menu_images_list) { /* Freeing already loaded pics */
		free_images_list (prefs_menu_images_list);
		prefs_menu_images_list = NULL;
	}
	
	/* How many activities do we have */
	
	nb_activities = sizeof (activities_names) / sizeof (activities_names[0]);
	
	ang_step = 360 / nb_activities;
	
	for (i = 0; i < nb_activities; i++) {
			
		new_image = (image_node *) malloc (sizeof(image_node));
			
		new_image->image_path = g_strdup_printf (
									"%s"G_DIR_SEPARATOR_S"%s.png",
									current_theme,
									activities_names[i]);
				
		new_image->image = gdk_pixbuf_new_from_file (new_image->image_path, &error);
			
		if (!new_image->image) {
			g_message ("failed to load image %s", new_image->image_path);
			return FALSE;
		}
		
		if (!i) {
			image_coord = 0; /* First picture at 0 degree */
			new_image->selected = TRUE; /* Is image selected */
		}
		else {
			image_coord = image_coord + ang_step; /* Changing image_coord */
			new_image->selected = FALSE; /* Is image selected */
		}
		
		new_image->image_label = g_strdup(gettext(activities_names[i]));
		
		new_image->xpos = new_image->x_offset = 0;
		
		new_image->ypos = new_image->y_offset = 0;
		
		new_image->width = gdk_pixbuf_get_width (new_image->image);
		
		new_image->height = gdk_pixbuf_get_height (new_image->image);
		
		new_image->alpha_level = 255; /* Alpha level to maximum */
		
		new_image->zoom_ratio = new_image->init_zoom_ratio = 1; /* Zoom ratio full size */
		
		new_image->visible = TRUE; /* Visible by default */
		
		new_image->busy = FALSE; /* Not busy by default */
		
		new_image->angle = image_coord + ANG_OFFSET; /* Initial menu position */
				
		if (new_image->angle > 360)
			new_image->angle = new_image->angle - 360; /* Staying between 0-360 */
		
		new_image->data = NULL;
		
		switch (i) {
			case 0 :
				new_image->image_link = (GtkFunction) activate_themes_menu;
				break;;
			case 1 :
				new_image->image_link = (GtkFunction) activate_anim_menu;
				break;;
			case 2 :
				new_image->image_link = (GtkFunction) activate_screensaver_menu;
				break;;
			case 3 :
				new_image->image_link = (GtkFunction) activate_main_menu;
				break;;
		}
		
		new_image->next_image = NULL;
		
		add_image_at_the_end (new_image, &prefs_menu_images_list);
	
	}
	
	return TRUE;
}

/* load_theme_menu
	Function which loads the theme selection menu's pictures in an image
	list memory structure */

gboolean
load_themes_menu (void) {
	gint i, image_coord = 0, ang_step;
	gint nb_activities;
	GError * error = NULL;
	image_node * new_image;
	GSList * themes_list = NULL;
	gchar * home_theme_dir = NULL, * default_theme_dir = NULL;
		
	remove_all_timers ();
	
	if (themes_menu_images_list) { /* Freeing already loaded pics */
		free_images_list (themes_menu_images_list);
		themes_menu_images_list = NULL;
	}
	
	/* Scanning for themes */
	
	default_theme_dir = g_build_path ( G_DIR_SEPARATOR_S,
							DATADIR,
							"getcontrol",
							"themes", NULL);
	
	scan_for_themes (&themes_list, default_theme_dir);
	
	if (default_theme_dir) g_free (default_theme_dir);
	
	home_theme_dir =  g_build_path ( G_DIR_SEPARATOR_S,
						g_get_home_dir (),
						".getcontrol",
						"themes",NULL);
	
	scan_for_themes (&themes_list, home_theme_dir);
	
	if (home_theme_dir)
		g_free(home_theme_dir);
	
	if (!themes_list) /* No valid theme found */
		return FALSE;
	
	/* How many activities do we have */
	
	nb_activities = g_slist_length (themes_list);
	
	ang_step = 360 / nb_activities;
	
	for (i = 0; i < nb_activities; i++) {
		
		xmlconfig * theme_config = NULL;
		
		gchar * theme_config_file = NULL;
		
		theme_config_file = g_build_path ( G_DIR_SEPARATOR_S,
								(gchar *)g_slist_nth_data (themes_list, i),
								"theme.xml", NULL);
		
		theme_config = xmlconfig_open(theme_config_file);
		
		if (!theme_config) {
			g_message ("Failed to load XML config file %s", theme_config_file);
			return FALSE;
		}
		
		new_image = (image_node *) malloc (sizeof(image_node));
			
		new_image->image_path = g_build_path ( G_DIR_SEPARATOR_S,
									(gchar *)g_slist_nth_data (themes_list, i),
									"theme.png", NULL);
		
		new_image->image = gdk_pixbuf_new_from_file (new_image->image_path, &error);
			
		if (!new_image->image) {
			g_message ("failed to load image %s", new_image->image_path);
			return FALSE;
		}
		
		if (!i) {
			image_coord = 0; /* First picture at 0 degree */
			new_image->selected = TRUE; /* Is image selected */
		}
		else {
			image_coord = image_coord + ang_step; /* Changing image_coord */
			new_image->selected = FALSE; /* Is image selected */
		}
		
		new_image->image_label = xmlconfig_get_content (
									theme_config,
									"/theme/theme_name");
		
		new_image->xpos = new_image->x_offset = 0;
		
		new_image->ypos = new_image->y_offset = 0;
		
		new_image->width = gdk_pixbuf_get_width (new_image->image);
		
		new_image->height = gdk_pixbuf_get_height (new_image->image);
		
		new_image->alpha_level = 255; /* Alpha level to maximum */
		
		new_image->zoom_ratio = new_image->init_zoom_ratio = 1; /* Zoom ratio full size */
		
		new_image->visible = TRUE; /* Visible by default */
		
		if ((current_theme) &&
			(g_strcasecmp (current_theme, (gchar *)g_slist_nth_data (themes_list, i))==0)) {
			g_message ("%s is busy",(gchar *)g_slist_nth_data (themes_list, i));
			new_image->busy = TRUE;
		}
		else
			new_image->busy = FALSE;
		
		new_image->angle = image_coord + ANG_OFFSET; /* Initial menu position */
				
		if (new_image->angle > 360)
			new_image->angle = new_image->angle - 360; /* Staying between 0-360 */
		
		new_image->image_link = (GtkFunction) activate_set_theme;
		
		new_image->data = (gchar *)g_slist_nth_data (themes_list, i);
		
		new_image->next_image = NULL;
		
		add_image_at_the_end (new_image, &themes_menu_images_list);
		
		if (theme_config)
			xmlconfig_close (theme_config);
	
	}
	
	return TRUE;
}

/* load_screensaver_menu
	Function which loads the screensaver configuration menu's pictures in
	an image list memory structure */

gboolean
load_screensaver_menu (void) {
	gint i, image_coord = 0, ang_step;
	gint nb_activities;
	GError * error = NULL;
	image_node * new_image;
	
	gchar * activities_names[] = {
		gettext_noop ("ss_short"),
		gettext_noop ("ss_long"),
		gettext_noop ("ss_none")
	};
	
	remove_all_timers ();
	
	if (screensaver_menu_images_list) { /* Freeing already loaded pics */
		free_images_list (screensaver_menu_images_list);
		screensaver_menu_images_list = NULL;
	}
	
	/* How many activities do we have */
	
	nb_activities = sizeof (activities_names) / sizeof (activities_names[0]);
	
	ang_step = 360 / nb_activities;
	
	for (i = 0; i < nb_activities; i++) {
			
		new_image = (image_node *) malloc (sizeof(image_node));
			
		new_image->image_path = g_strdup_printf (
									"%s"G_DIR_SEPARATOR_S"%s.png",
									current_theme,
									activities_names[i]);
				
		new_image->image = gdk_pixbuf_new_from_file (new_image->image_path, &error);
			
		if (!new_image->image) {
			g_message ("failed to load image %s", new_image->image_path);
			return FALSE;
		}
		
		if (!i) {
			image_coord = 0; /* First picture at 0 degree */
			new_image->selected = TRUE; /* Is image selected */
		}
		else {
			image_coord = image_coord + ang_step; /* Changing image_coord */
			new_image->selected = FALSE; /* Is image selected */
		}
		
		new_image->image_label = g_strdup(gettext(activities_names[i]));
		
		new_image->xpos = new_image->x_offset = 0;
		
		new_image->ypos = new_image->y_offset = 0;
		
		new_image->width = gdk_pixbuf_get_width (new_image->image);
		
		new_image->height = gdk_pixbuf_get_height (new_image->image);
		
		new_image->alpha_level = 255; /* Alpha level to maximum */
		
		new_image->zoom_ratio = new_image->init_zoom_ratio = 1; /* Zoom ratio full size */
		
		new_image->visible = TRUE; /* Visible by default */
		
		new_image->angle = image_coord + ANG_OFFSET; /* Initial menu position */
				
		if (new_image->angle > 360)
			new_image->angle = new_image->angle - 360; /* Staying between 0-360 */
		
		new_image->image_link = (GtkFunction) activate_set_ss_delay;
		
		switch (i) {
			case 0 :
				new_image->data = g_strdup("30000");
				break;;
			case 1 :
				new_image->data = g_strdup("600000");
				break;;
			case 2 :
				new_image->data = g_strdup("0");
				break;;
		}
		
		if (screensaver_delay == atoi (new_image->data) )
			new_image->busy = TRUE;
		else
			new_image->busy = FALSE; /* Not busy by default */
		
		new_image->next_image = NULL;
		
		add_image_at_the_end (new_image, &screensaver_menu_images_list);
	
	}
	
	return TRUE;
}

/* load_anim_prefs_menu
	Function which loads the animation speed settings menu's pictures in
	an image list memory structure */
	
gboolean
load_anim_prefs_menu (void) {
	gint i, image_coord = 0, ang_step;
	gint nb_activities;
	GError * error = NULL;
	image_node * new_image;
	
	gchar * activities_names[] = {
		gettext_noop ("anim_slow"),
		gettext_noop ("anim_average"),
		gettext_noop ("anim_fast")
	};
	
	remove_all_timers ();
	
	if (anim_menu_images_list) { /* Freeing already loaded pics */
		free_images_list (anim_menu_images_list);
		anim_menu_images_list = NULL;
	}
	
	/* How many activities do we have */
	
	nb_activities = sizeof (activities_names) / sizeof (activities_names[0]);
	
	ang_step = 360 / nb_activities;
	
	for (i = 0; i < nb_activities; i++) {
			
		new_image = (image_node *) malloc (sizeof(image_node));
			
		new_image->image_path = g_strdup_printf (
									"%s"G_DIR_SEPARATOR_S"%s.png",
									current_theme,
									activities_names[i]);
				
		new_image->image = gdk_pixbuf_new_from_file (new_image->image_path, &error);
			
		if (!new_image->image) {
			g_message ("failed to load image %s", new_image->image_path);
			return FALSE;
		}
		
		if (!i) {
			image_coord = 0; /* First picture at 0 degree */
			new_image->selected = TRUE; /* Is image selected */
		}
		else {
			image_coord = image_coord + ang_step; /* Changing image_coord */
			new_image->selected = FALSE; /* Is image selected */
		}
		
		new_image->image_label = g_strdup(gettext(activities_names[i]));
		
		new_image->xpos = new_image->x_offset = 0;
		
		new_image->ypos = new_image->y_offset = 0;
		
		new_image->width = gdk_pixbuf_get_width (new_image->image);
		
		new_image->height = gdk_pixbuf_get_height (new_image->image);
		
		new_image->alpha_level = 255; /* Alpha level to maximum */
		
		new_image->zoom_ratio = new_image->init_zoom_ratio = 1; /* Zoom ratio full size */
		
		new_image->visible = TRUE; /* Visible by default */
		
		new_image->busy = FALSE; /* Not busy by default */
		
		new_image->angle = image_coord + ANG_OFFSET; /* Initial menu position */
				
		if (new_image->angle > 360)
			new_image->angle = new_image->angle - 360; /* Staying between 0-360 */
		
		new_image->image_link = (GtkFunction) activate_set_anim_speed;
		
		switch (i) {
			case 0 :
				new_image->data = g_strdup("100");
				break;;
			case 1 :
				new_image->data = g_strdup("50");
				break;;
			case 2 :
				new_image->data = g_strdup("15");
				break;;
		}
		
		if (animation_frame_delay == atoi (new_image->data) )
			new_image->busy = TRUE;
		else
			new_image->busy = FALSE; /* Not busy by default */
		
		new_image->next_image = NULL;
		
		add_image_at_the_end (new_image, &anim_menu_images_list);
	
	}
	
	return TRUE;
}



/* ===================================================== */

/* =================Activation functions================ */

gboolean
activate_prefs_menu (gpointer data) 
{
	if (load_prefs_menu ()) {
		current_menu_images_list = prefs_menu_images_list;
		current_menu = PREFS_MENU;
	}
	
	draw_menu (current_menu_images_list);
	
	return TRUE;
	
}

gboolean
activate_themes_menu (gpointer data) 
{
	if (load_themes_menu ()) {
		current_menu_images_list = themes_menu_images_list;
		current_menu = THEMES_MENU;
	}
	
	draw_menu (current_menu_images_list);
	
	return TRUE;
	
}

gboolean
activate_screensaver_menu (gpointer data) 
{
	if (load_screensaver_menu ()) {
		current_menu_images_list = screensaver_menu_images_list;
		current_menu = SS_MENU;
	}
	
	draw_menu (current_menu_images_list);
	
	return TRUE;
	
}

gboolean
activate_anim_menu (gpointer data) 
{
	if (load_anim_prefs_menu ()) {
		current_menu_images_list = anim_menu_images_list;
		current_menu = ANIM_MENU;
	}
	
	draw_menu (current_menu_images_list);
	
	return TRUE;
	
}

gboolean
activate_main_menu (gpointer data) 
{
	
	if (load_main_menu ()) {
		current_menu_images_list = main_menu_images_list;
		current_menu = MAIN_MENU;
	}
	
	draw_menu (current_menu_images_list);
	
	return TRUE;
	
}

gboolean
activate_set_theme (gpointer data)
{
	g_message("setting theme %s", (gchar *) data);
	
	gconf_client_set_string ( conf_client, THEME_PATH, (gchar *) data, NULL);
	
	activate_prefs_menu (NULL);
	
	return TRUE;
}

gboolean
activate_set_anim_speed (gpointer data)
{
	g_message("setting anim speed %s", (gchar *) data);
	
	gconf_client_set_int ( conf_client, ANIM_DELAY_PATH, atoi((gchar *) data), NULL);
	
	activate_prefs_menu (NULL);
	
	return TRUE;
}

gboolean
activate_set_ss_delay(gpointer data)
{
	g_message("setting ss delay %s", (gchar *) data);
	
	gconf_client_set_int ( conf_client, SS_DELAY_PATH, atoi((gchar *) data), NULL);
	
	activate_prefs_menu (NULL);
	
	return TRUE;
}
gboolean
activate_dummy_menu (gpointer data)
{
	g_message("dummy menu");
	
	return TRUE;
}

/* ===================================================== */

/* ==================Drawing functions================== */

/* draw_menu
	Function which draw the menu contained in the passed images_list
*/

gboolean
draw_menu (gpointer data)
{
	image_node * draw_images_list = NULL;
	image_node * images_list = (image_node *) data;
	
	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) /* Images list not initialized */
		return TRUE;
	
	/* Copying the background picture in frame */
	
	gdk_pixbuf_copy_area (	background,
							0, 0,
							draw_width, draw_height,
							frame, 0, 0);


	while (draw_images_list) {	

		/* Calculating x, y positions for each picture */
		
		draw_images_list->xpos = xmid + ( radius * cos ( 2.0 * M_PI * draw_images_list->angle / 360) )
								- (draw_images_list->width *  draw_images_list->init_zoom_ratio / 2.0);
		draw_images_list->ypos = ymid - ( radius * sin ( 2.0 * M_PI * draw_images_list->angle / 360) )
								- (draw_images_list->height *  draw_images_list->init_zoom_ratio / 2.0);
		
		draw_images_list->x_offset = 0;
		draw_images_list->y_offset = 0;
		
		draw_images_list->zoom_ratio = draw_images_list->init_zoom_ratio;
		
		draw_images_list->alpha_level = 255;
		
		draw_images_list->visible = TRUE; /* Visible */
		
		draw_images_list = draw_images_list->next_image;
	}
	
	/* Now drawing */
	
	draw_images (images_list);
	
	current_status = IDLE;
	
	if (!select_timeout_id) /* Installing timer for selection */
		select_timeout_id = gtk_timeout_add (	selection_delay,
												menu_selection_manager,
												images_list);
		
	if ( (!screen_saver_timeout_id) && (screensaver_delay) ) /* Installing timer for screen saver */
		screen_saver_timeout_id = gtk_timeout_add (
										screensaver_delay + selection_delay,
										screen_saver_manager,
										images_list);
	
	return TRUE;
}

/* menu_rotation_manager
	Function which rotates menu with an animation
*/

gboolean
menu_rotation_manager (gpointer data)
{
	gint rotate, alpha_level;
	guint nb_items = 0;
	gdouble zoom_ratio;
	image_node * draw_images_list = NULL;
	image_node * images_list = (image_node *) data;
	
	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if ( (!draw_images_list) || !(draw_images_list->next_image) ) /* Images list not initialized */
		return FALSE;
	
	if ( !draw_timeout_id ) { /* No timer installed */
		
		/* Set the lock while rotating */
		
		current_status = ROTATING;
		
		remove_all_timers ();
		
		while (draw_images_list) { /* Counting the number of items */
			nb_items++;
			draw_images_list = draw_images_list->next_image;
		}
	
		draw_images_list = images_list; /* Going back to the images_list */
	
		/* Setting the number of degrees to move */
		
		left_to_move = total_move = 360 / nb_items;
		
		/* Telling what we are doing */
		
		if (direction == LEFT)		
			gnome_appbar_set_status(GNOME_APPBAR (appbar), _("Rotating <-"));
		else 
			gnome_appbar_set_status(GNOME_APPBAR (appbar), _("Rotating ->"));
		
		/* Installing timer for animation */
		
		if (!draw_timeout_id)
			draw_timeout_id = gtk_timeout_add (animation_frame_delay, menu_rotation_manager, images_list);
	
	}
	
	if (reload_timers) { /* Timers have changed */
		
		if (draw_timeout_id)
			gtk_timeout_remove (draw_timeout_id);
		
		draw_timeout_id = gtk_timeout_add (animation_frame_delay, menu_rotation_manager, images_list);
		
		reload_timers = FALSE;
	}
		
	if (!left_to_move) { /* We finnished our move */
		
		if (draw_timeout_id)
			gtk_timeout_remove ( draw_timeout_id); /* Disabling timer */
		
		draw_timeout_id = 0; /* Resetting draw timer */
		
		if (direction == LEFT)
			select_previous (draw_images_list);
		else
			select_next (draw_images_list);
		
		if (order_queue) {
			
			switch (order_queue->order_type) {
			
				case ROTATE_ORDER :
					direction = order_queue->direction;
					unstack_order (&order_queue);
					menu_rotation_manager (current_menu_images_list);
					break;;
				case ACTIVATE_ORDER :
					while (order_queue)
						unstack_order (&order_queue);
					draw_images_list = current_menu_images_list;
					while ( !(draw_images_list->selected) && (draw_images_list) )
						draw_images_list = draw_images_list->next_image;
					draw_images_list->image_link (draw_images_list->data);
					break;;
			}
					
			
		}
		else {
			
			if (!select_timeout_id) /* Installing timer for menu selection */
				select_timeout_id = gtk_timeout_add (selection_delay, menu_selection_manager, images_list);
		
			if ( (!screen_saver_timeout_id) && (screensaver_delay) )/* Installing timer for screen saver */
				screen_saver_timeout_id = gtk_timeout_add (
										screensaver_delay + selection_delay,
										screen_saver_manager,
										images_list);
	
		
			gnome_appbar_set_status(GNOME_APPBAR (appbar), _("Done rotation")); /* Telling status */
		
			current_status = IDLE;
		
		}
		return TRUE;
	}
	
	if (direction == LEFT) {
		if (left_to_move <= ROT_OFFSET) {
			alpha_level = 255;
			zoom_ratio = 1; /* On the last move we finnish on original zoom_ratio and alpha */
			rotate = left_to_move; /* Needed rotation is smaller than the preferred one */
		}
		else {
			alpha_level = 255 - ( ( ( cos ( ( ( left_to_move * M_PI * 2.0 ) / total_move ) + M_PI ) + 1 ) / 2 ) * ALPHA_THRESOLD );
			
			/* Here we calculate a zoom_ratio modificator which will be used to impact the initial zoom
			ratio. This way the initial zoom ratio can change the modificator will keep a coherent
			aspect. This ratio will vary from 1 to 2 following cosinus graph from Pi on a 2 Pi period */
			
			zoom_ratio = ( cos ( ( ( left_to_move * M_PI * 2.0 ) / total_move ) + M_PI ) + 1 ) / 2.0 + 1;
			
			rotate = ROT_OFFSET;
		}
	}
	else {
		if (left_to_move <= ROT_OFFSET) {
			alpha_level = 255;
			zoom_ratio = 1; /* On the last move we finnish on original zoom_ratio and alpha */
			rotate = 0 - left_to_move; /* Needed rotation is smaller than the preferred one */
		}
		else {
			alpha_level = 255 - ( ( ( cos ( ( ( left_to_move * M_PI * 2.0 ) / total_move ) + M_PI ) + 1 ) / 2 ) * ALPHA_THRESOLD );
			
			/* Here we calculate a zoom_ratio modificator which will be used to impact the initial zoom
			ratio. This way the initial zoom ratio can change the modificator will keep a coherent
			aspect. This ratio will vary from 1 to 2 following cosinus graph from Pi on a 2 Pi period */
			
			zoom_ratio = ( cos ( ( ( left_to_move * M_PI * 2.0 ) / total_move ) + M_PI ) + 1 ) / 2.0 + 1;
			rotate = 0 - ROT_OFFSET;
		}
	}

	while (draw_images_list) {
		
		draw_images_list->angle = draw_images_list->angle + rotate; /* Changing image angle */
		
		if (draw_images_list->angle >= 360) /* We stay between 0-360 */
			draw_images_list->angle = draw_images_list->angle - 360;
		if (draw_images_list->angle <= 0) /* We stay between 0-360 */
			draw_images_list->angle = 360 + draw_images_list->angle;

		draw_images_list->zoom_ratio = draw_images_list->init_zoom_ratio / zoom_ratio;
		
		draw_images_list->xpos = xmid + ( radius * cos ( 2.0 * M_PI * draw_images_list->angle / 360) )
								- ( draw_images_list->width * draw_images_list->zoom_ratio / 2.0);
		draw_images_list->ypos = ymid - ( radius * sin ( 2.0 * M_PI * draw_images_list->angle / 360) )
								- (draw_images_list->height * draw_images_list->zoom_ratio / 2.0);
		
		draw_images_list->x_offset = 0;
		draw_images_list->y_offset = 0;
		
		draw_images_list->alpha_level = alpha_level;
		
		draw_images_list->visible = TRUE; /* Visible */
		
		draw_images_list = draw_images_list->next_image;
	}
	
	if (direction == LEFT)
		left_to_move = left_to_move - rotate; /* Removing the rotation we've done */
	else
		left_to_move = left_to_move - (0 -rotate); /* Removing the rotation we've done */
	
	/* Copying the background picture in frame */
	
	gdk_pixbuf_copy_area (	background,
							0, 0,
							draw_width, draw_height,
							frame, 0, 0);
	
	draw_images (images_list);
	
	return TRUE;
}

/* menu_selection_manager
	Function managing the selection of the top menu
*/

gboolean
menu_selection_manager (gpointer data)
{
	image_node * draw_images_list = NULL;
	image_node * images_list = (image_node *) data;
	gboolean busy_anim = FALSE;
	
	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) /* Images list not initialized */
		return TRUE;
	
	current_status = SELECTING;
		
	remove_all_timers ();
	
	/* Copying the background picture in frame */
	
	gdk_pixbuf_copy_area (	background,
							0, 0,
							draw_width, draw_height,
							frame, 0, 0);


	while (draw_images_list) {
		
		draw_images_list->x_offset = 0;
		draw_images_list->y_offset = 0;
		
		draw_images_list->zoom_ratio = draw_images_list->init_zoom_ratio;
		
		if (draw_images_list->selected) {
			draw_images_list->alpha_level = 255;
			selected_label = g_strdup (draw_images_list->image_label);
		}
		else
			draw_images_list->alpha_level = 140;
		
		draw_images_list->visible = TRUE; /* Visible */
		
		if (draw_images_list->busy)
			busy_anim = TRUE;
		
		draw_images_list = draw_images_list->next_image;
	}
	
	/* Now drawing */
	
	current_status = SELECTED;
	
	draw_images (images_list);
	
	if ( (busy_anim) && (!busy_draw_timeout_id) ) {
				/* Installing draw timer */
				busy_iteration = 0;
				busy_draw_timeout_id = gtk_timeout_add ( animation_frame_delay,
										busy_animation_manager, images_list);
	}
	
	if ( (!screen_saver_timeout_id) && (screensaver_delay) ) /* Installing timer for screen saver */
		screen_saver_timeout_id = gtk_timeout_add (
										screensaver_delay + selection_delay,
										screen_saver_manager,
										images_list);
	
	return TRUE;
}



gint busy_offsets [] = {
						-2,
						-4,
						-8,
						-16,
						-32,
						-40,
						-32,
						-16,
						-4,
						0,
						-2,
						-4,
						-8,
						-15,
						-8,
						-4,
						0 };
					
gboolean busy_tempo = FALSE;

/* busy_animation_manager
	Function managing the busy menus animation
*/

gboolean
busy_animation_manager (gpointer data)
{
	image_node * draw_images_list = NULL;
	image_node * images_list = (image_node *) data;
	
	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (!draw_images_list) /* Images list not initialized */
		return TRUE;
	
	if (busy_tempo) {
		if (busy_draw_timeout_id)
			gtk_timeout_remove (busy_draw_timeout_id);
	
		/* Installing draw timer */
		busy_draw_timeout_id = gtk_timeout_add ( animation_frame_delay,
									busy_animation_manager, images_list);
		busy_tempo = FALSE;
		return TRUE;
	}
	
	/* Copying the background picture in frame */
	
	gdk_pixbuf_copy_area (	background,
							0, 0,
							draw_width, draw_height,
							frame, 0, 0);


	while (draw_images_list) {	

		if (draw_images_list->busy) {
			draw_images_list->y_offset = busy_offsets[busy_iteration];
		}
			
		draw_images_list->visible = TRUE;
			
		draw_images_list = draw_images_list->next_image;
	}
	
	busy_iteration ++;
	
	if (busy_iteration > 16) {
		busy_iteration = 0;
		if (busy_draw_timeout_id)
			gtk_timeout_remove (busy_draw_timeout_id);
	
		/* Installing draw timer */
		busy_draw_timeout_id = gtk_timeout_add ( 1000,
									busy_animation_manager, images_list);
		busy_tempo = TRUE;
	}
	
	/* Now drawing */
	
	draw_images (images_list);
	
	return TRUE;
}


/* screen_saver_manager
	Function managing the screensaver
*/

gint frame_num;

gboolean
screen_saver_manager (gpointer data)
{
	gdouble f;
	guint i = 0;
	image_node * draw_images_list = NULL;
	guint nb_items = 0;
	image_node * images_list = (image_node *) data;
	
	/* Copying the head pointer of the images list */
	
	draw_images_list = images_list;
	
	if (screen_saver_timeout_id) {
		remove_all_timers ();
	}
	
	if (!ss_draw_timeout_id) { /* First call */
		
		current_status = SCREEN_SAVING;
		
		/* Initializing background slideshow */
		
		if (!ss_bg_list)
			init_ss_bg ();
		
		/* If no picture available no switching */
		
		if (g_slist_length(ss_bg_list))
			ss_bg_switch_timeout_id = gtk_timeout_add (
										bg_switch_delay, ss_bg_switch, NULL);
		
		/* Installing timer for animation */
		
		ss_draw_timeout_id = gtk_timeout_add (
								animation_frame_delay,
								screen_saver_manager, images_list);
	}
	
	if (reload_timers) { /* Timers have changed */
		
		if (ss_draw_timeout_id)
			gtk_timeout_remove (ss_draw_timeout_id);
		
		ss_draw_timeout_id = gtk_timeout_add (
								animation_frame_delay,
								screen_saver_manager, images_list);
		
		reload_timers = FALSE;
	}
	
	gdk_pixbuf_copy_area (	background,
							0, 0,
							draw_width, draw_height,
							frame, 0, 0);

	f = (gdouble) (frame_num % 60) / 60;
	
	while (draw_images_list) { /* Counting the number of items */
		nb_items++;
		draw_images_list = draw_images_list->next_image;
	}
	
	draw_images_list = images_list; /* Going back to the images_list */

	while (draw_images_list) {
		gdouble ang, r, k;

		ang = 2.0 * M_PI * (gdouble) i / nb_items - f * 2.0 * M_PI;

		r = radius + (radius / 3.0) * sin (f * 2.0 * M_PI);

		draw_images_list->xpos = floor (xmid + r * cos (ang) - draw_images_list->width / 2.0 + 0.5);
		draw_images_list->ypos = floor (ymid + r * sin (ang) - draw_images_list->height / 2.0 + 0.5);

		draw_images_list->x_offset = 0;
		draw_images_list->y_offset = 0;
		
		k = (i & 1) ? sin (f * 2.0 * M_PI) : cos (f * 2.0 * M_PI);
		k = 2.0 * k * k;
		k = MAX (0.25, k);

		draw_images_list->zoom_ratio = k;
		
		draw_images_list->alpha_level = ((i & 1)
					       ? MAX (127, fabs (255 * sin (f * 2.0 * M_PI)))
					       : MAX (127, fabs (255 * cos (f * 2.0 * M_PI))));

		draw_images_list->visible = TRUE;
		
		draw_images_list = draw_images_list->next_image;
		
		i++;
	}
	
	draw_images (images_list);
	
	frame_num++;
	
	return TRUE;
}

/* Function loading backgrounds for screensaving slideshow */

gboolean
init_ss_bg (void)
{
	GDir * theme_dir = NULL;
	gchar * file_name = NULL;
	
	
	ss_bg_list = g_slist_alloc ();
	
	/* Opening dir */
	
	theme_dir = g_dir_open (current_theme, 0, NULL);
	
	if (!theme_dir) { /* Failed to open dir */
		g_message ("unable to open dir %s", current_theme);
		return;
	}
	
	/* For each file */
	
	while (file_name = (gchar *) g_dir_read_name (theme_dir)) {
		gchar * l_file_name = NULL;
		
		l_file_name = g_build_path ( G_DIR_SEPARATOR_S,
						current_theme, file_name, NULL);
		
		/* Searching for "screensaver*" */
		
		if (g_strrstr (file_name, "ss_bg")) {
			GdkPixbuf * new_pixbuf = NULL;
			GError * error = NULL;
			
			new_pixbuf = gdk_pixbuf_new_from_file (l_file_name, &error);

			ss_bg_list = g_slist_append (ss_bg_list, new_pixbuf);
			
		}
			
		
		if (l_file_name)
			g_free (l_file_name);		
	}
	
	g_dir_close (theme_dir);
	
	if ( g_slist_length(ss_bg_list) == 0 )
		g_message ("no screensaver background found. using default background");
	
	return TRUE;
}

/* Function changing current background while screensaving */

gboolean
ss_bg_switch (gpointer data)
{
	gdouble width_ratio_scale, height_ratio_scale;
	GdkPixbuf * double_buffer = NULL, * backup = NULL, * ss_pixbuf = NULL;
	
	double_buffer = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
									FALSE, 8, draw_width,
									draw_height);
	
	if (double_buffer) { /* Has the background buffer been created already */
	
		ss_pixbuf = (GdkPixbuf *) g_slist_nth_data (ss_bg_list, ss_bg_id);
		
		/* Calculating ratio for each background */
		
		width_ratio_scale = (gdouble) draw_width /
							(gdouble) gdk_pixbuf_get_width (ss_pixbuf);
		height_ratio_scale = (gdouble) draw_height /
							(gdouble) gdk_pixbuf_get_height (ss_pixbuf);
	
		/* Drawing & scaling our background */
	
		gdk_pixbuf_composite (
			ss_pixbuf,
			double_buffer,
			0, 0,
			draw_width, draw_height,
			0, 0,
			width_ratio_scale, height_ratio_scale,
			GDK_INTERP_BILINEAR,
			255);
		
		ss_bg_id++;
		
		if (ss_bg_id >= g_slist_length(ss_bg_list))
			ss_bg_id = 1;
		
		g_message ("ss_bg_id is now %d", ss_bg_id);
	}
	
	backup = background;
	background = gdk_pixbuf_copy (double_buffer);
	
	g_object_unref (backup);
	
	return TRUE;
}

gboolean
destroy_ss_bg_list ()
{
	gint i, nb_elem;
	
	nb_elem = g_slist_length (ss_bg_list);
	
	for ( i=0; i<nb_elem; i++) {
		g_object_unref ( (GdkPixbuf *) g_slist_nth_data (ss_bg_list, i));
	}

	g_slist_free (ss_bg_list);
	
	ss_bg_list = NULL;

	return TRUE;	
	
}

/* ===================================================== */


/* ==================Orders Managers==================== */

gboolean
stack_order (order_node ** l_order_queue, gboolean l_direction, guint order_type)
{
	order_node * new_order = (order_node *) malloc (sizeof(order_node));
	order_node * back_order = NULL;
	
	back_order = *l_order_queue;
	
	new_order->order_type = order_type;
	
	new_order->direction = l_direction;
	
	new_order->next_order = NULL;
	
	if (!back_order) {
		*l_order_queue = new_order;
		return TRUE;
	}

	while (back_order->next_order)
		back_order = back_order->next_order;
	
	back_order->next_order = new_order;
	
	return TRUE;
}

gboolean
unstack_order (order_node ** l_order_queue)
{
	
	order_node * new_order = NULL;
	
	new_order = *l_order_queue;
	
	*l_order_queue = new_order->next_order;
	
	g_free (new_order);
	
	return TRUE;
	
}

/* ===================================================== */

/* ==================Scan for Themes==================== */

/* scan_for_themes
	Function scanning for valid themes
*/

void
scan_for_themes (GSList ** themes_list, gchar * theme_path)
{
	GDir * theme_dir = NULL;
	gchar * file_name = NULL;
	
	/* Opening dir */
	
	theme_dir = g_dir_open (theme_path, 0, NULL);
	
	if (!theme_dir) { /* Failed to open dir */
		g_message ("unable to open dir %s", theme_path);
		return;
	}
	
	/* For each file */
	
	while (file_name = (gchar *) g_dir_read_name (theme_dir)) {
		gchar * l_file_name = NULL;
		
		l_file_name = g_build_path ( G_DIR_SEPARATOR_S,
						theme_path, file_name, NULL);
		
		if (g_file_test (l_file_name, G_FILE_TEST_IS_DIR)) {
			/* File is a directory */
			
			if (check_theme_validity (l_file_name)) /* Checking theme */
				*themes_list = g_slist_append (
							*themes_list,
							(gchar *) g_strdup (l_file_name));
		}
		
		if (l_file_name)
			g_free (l_file_name);		
	}
	
	g_dir_close (theme_dir);
	
}

/* check_theme_validity
	Function checking if a supplied theme is valid or not
*/

gboolean
check_theme_validity (gchar * theme_path)
{
	gchar * files_list[] = {
		"anim_average.png",
		"anim_fast.png",
		"anim_slow.png",
		"audio.png",
		"background.jpg",
		"back.png",
		"divx.png",
		"dvd.png",
		"mame.png",
		"photos.png",
		"prefs_anim.png",
		"prefs.png",
		"prefs_screensaver.png",
		"prefs_themes.png",
		"ss_long.png",
		"ss_none.png",
		"ss_short.png",
		"theme.png",
		"from_cd.png",
		"from_hd.png",
		"from_nfs.png",
		"from_stream.png",
		"theme.xml"
	};
	
	guint nb_files = sizeof (files_list) / sizeof (files_list[0]);
	guint i;
	
	for (i = 0; i < nb_files; i++) {
	
		gchar * file_path = NULL;
		
		file_path = g_build_path ( G_DIR_SEPARATOR_S,
							theme_path,
							files_list[i], NULL);
				
		if (!g_file_test (file_path, G_FILE_TEST_EXISTS)) {
			g_message ("file %s is missing for theme %s", file_path, theme_path);
			if (file_path)
				g_free (file_path);
			return FALSE;
		}
		
		if (file_path)
			g_free (file_path);
		
	}
	
	return TRUE;
	
}

/* ===================================================== */
