/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 /* $Id: menu.h,v 1.21 2003/01/11 23:10:19 dolphy Exp $ */
 
#ifndef MENU_H
#define MENU_H

/* ==========Order Queue====== */

typedef struct order_q { /* Chained list containing orders */
	guint order_type;
	gboolean direction; /* Direction of queued rotation */
	struct order_q * next_order; /* Pointer to next order */
} order_node;

extern order_node * order_queue;

enum {
	ROTATE_ORDER,
	ACTIVATE_ORDER
};

/* =========================== */

/* ===========Timers========== */

/* Delay between frames when animating */

guint animation_frame_delay;

/* Delay before saving screen */

guint screensaver_delay;

/* Delay before selecting menu */ 

guint selection_delay;

/* Global timer id */

guint select_timeout_id, screen_saver_timeout_id;

/* Animation timers */

guint ss_draw_timeout_id, draw_timeout_id, busy_draw_timeout_id;

guint ss_bg_switch_timeout_id;

/* Reloading timer flag */

gboolean reload_timers;

/* =========================== */

/* Rotation directions */

enum { 
	LEFT,
	RIGHT
};

extern gboolean direction;

/* Background image and its dimensions */

extern GdkPixbuf * background, * background_orig;

extern gint back_width, back_height, xmid, ymid, radius, draw_width, draw_height;

/* Number of degrees remaining to rotate */

extern gint left_to_move, total_move;

/* Defining our statuses */

enum {
	IDLE,
	ROTATING,
	SELECTING,
	SELECTED,
	SCREEN_SAVING,
	ACTIVATING,
	V_PLAYING,
	A_PLAYING
};

/* Global info on what we are doing */

extern gint current_status;

/* Defining our menus */

enum {
	MAIN_MENU,
	PREFS_MENU,
	THEMES_MENU,
	SS_MENU,
	ANIM_MENU,
	DIVX_MENU,
	AUDIO_MENU,
	STREAMS_MENU,
	BUDDIES_MENU,
};

/* Global info on which menu we are */

extern guint current_menu;

/* Current screensaver background id */

extern guint ss_bg_id;

/* Global info on the current theme */

extern gchar * current_theme;

/* Current theme font markup */

extern gchar * font_markup;

/* Label of the selected menu */

extern gchar * selected_label;

/* ============Menus============ */

extern image_node * main_menu_images_list, * prefs_menu_images_list;
extern image_node * themes_menu_images_list, * screensaver_menu_images_list;
extern image_node * anim_menu_images_list, * current_menu_images_list;

extern GSList * ss_bg_list;

/* ==================Loading functions================== */

/* load_config
	Function which loads config information from Gconf
*/

gboolean load_config (void);

/* load_background
	Function which loads the background pictures and draw it scaled in the
	background buffer for next draw event */

gboolean load_background (void);

/* load_main_menu
	Function which loads the main menu's pictures in an image list memory
	structure */

gboolean load_main_menu (void);

/* load_prefs_menu
	Function which loads the preferences menu's pictures in an image list
	memory structure */

gboolean load_prefs_menu (void);

/* load_theme_menu
	Function which loads the theme selection menu's pictures in an image
	list memory structure */

gboolean load_themes_menu (void);

/* load_screensaver_menu
	Function which loads the screensaver configuration menu's pictures in
	an image list memory structure */

gboolean load_screensaver_menu (void);

/* load_anim_prefs_menu
	Function which loads the animation speed settings menu's pictures in
	an image list memory structure */
	
gboolean load_anim_prefs_menu (void);





/* ===================================================== */

/* =================Activation functions================ */

gboolean activate_prefs_menu (gpointer data);
gboolean activate_themes_menu (gpointer data);
gboolean activate_screensaver_menu (gpointer data);
gboolean activate_anim_menu (gpointer data);
gboolean activate_main_menu (gpointer data);
gboolean activate_set_theme (gpointer data);
gboolean activate_set_anim_speed (gpointer data);
gboolean activate_set_ss_delay (gpointer data);
gboolean activate_dummy_menu (gpointer data);
gboolean activate_divx_menu (gpointer data);
gboolean activate_audio_menu (gpointer data);

gboolean activate_divx_from_cd (gpointer data);
gboolean activate_divx_from_hd (gpointer data);
gboolean activate_divx_from_nfs (gpointer data);

gboolean activate_audio_from_stream (gpointer data);
gboolean activate_audio_from_cd (gpointer data);
gboolean activate_audio_from_hd (gpointer data);
gboolean activate_audio_from_nfs (gpointer data);

/* Restore functions */

gboolean restore_divx (gpointer data);
gboolean restore_audio (gpointer data);

/* ===================================================== */

/* ==================Drawing functions================== */

/* Draw main menu */

gboolean draw_menu (gpointer data);

/* Taking care of rotating the main menu */

gboolean menu_rotation_manager (gpointer data);

/* Taking care of animating menu selection */

gboolean menu_selection_manager (gpointer data);

/* busy_animation_manager
	Function managing the busy menus animation
*/

gboolean busy_animation_manager (gpointer data);

/* Taking care of screen saving */

gboolean screen_saver_manager (gpointer data);

/* Screensaver background switcher */

gboolean init_ss_bg (void);
gboolean ss_bg_switch (gpointer data);

/* ===================================================== */

/* ==================Orders Managers==================== */

gboolean stack_order (order_node ** l_order_queue, gboolean l_direction, guint order_type);

gboolean unstack_order (order_node ** l_order_queue);

/* ===================================================== */

/* ==================Scan for Themes==================== */

void scan_for_themes (GSList ** themes_list, gchar * theme_path);

gboolean check_theme_validity (gchar * theme_path);

/* ===================================================== */
#endif
